/*	calendar - calendaring and scheduling application reference implementation
 *  announce.c - when the time is up for an alarm event,the announce application 
 *              will be used to send dbus message to GPE application launcher.
 *
 *  Authors: YANG Lin <lin.yang@orange-ftgroup.com> 
 *  
 *  This software and associated documentation files (the "Software") 
 *  are copyright (C) 2005 LiPS Linux Phone Standards Forum [FranceTelecom] 
 *  All Rights Reserved. 
 *
 *  A copyright license is hereby granted for redistribution and use of 
 *  the Software in source and binary forms, with or without modification, 
 *  provided that the following conditions are met: 
 *  - Redistributions of source code must retain the above copyright notice, 
 *  this copyright license and the following disclaimer. 
 *  - Redistributions in binary form must reproduce the above copyright 
 *   notice, this copyright license and the following disclaimer in the 
 *  documentation and/or other materials provided with the distribution. 
 *  - Neither the name of LiPS  nor the names of its Members may be used 
 *  to endorse or promote products derived from the Software without 
 *  specific prior written permission. 
 *
 *  A patent license for any Necessary Claims owned by Members of LiPS Forum 
 *  to make, have made, use, import, offer to sell, lease and sell or otherwise 
 *  distribute any implementation compliant with the any specification adopted 
 *  by the LiPS Forumcan be obtained from the respective Members on reasonable 
 *  and non-discriminatory terms and conditions and under reciprocity, as 
 *  regulated in more detail in the Internal Policy of the LiPS Forum. 
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDER, ITS MEMBERS AND CONTRIBUTORS 
 *  "AS IS", AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
 *  THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE 
 *  AND NON-INFRINGEMENT ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER, 
 *  ITS MEMBERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
 *  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; 
 *  OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 *  WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 *  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
 *  POSSIBILITY OF SUCH DAMAGE. 
 */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <dbus/dbus-glib-lowlevel.h>
#include <dbus/dbus-glib.h>
#include <dbus/dbus-glib-bindings.h>



#define ANNOUNCE_SERVICE          "org.gpe.applauncher.msg"
#define ANNOUNCE_PATH             "/org/gpe/applauncher/msg"
#define ANNOUNCE_INTERFACE        "org.gpe.applauncher.msg"
#define ANNOUNCE_OBJECT           "/org/gpe/applauncher/msg"


int
main (int argc, char *argv[])
{
#if 0
  char *announcetext;
  GtkWidget *window;
  GtkWidget *frame;
  GtkWidget *label;


  gtk_set_locale ();
  gtk_init (&argc, &argv);

  announcetext = argv[1];
  window = gtk_dialog_new ();
  gtk_window_set_position (GTK_WINDOW (window), GTK_WIN_POS_CENTER);
  gtk_window_set_modal (GTK_WINDOW (window), TRUE);

  frame = gtk_frame_new (NULL);
  gtk_box_pack_start (GTK_BOX (GTK_DIALOG (window)->vbox), frame, TRUE, TRUE,
                      3);

  label = gtk_label_new (announcetext);
  gtk_container_add (GTK_CONTAINER (frame), label);

  g_signal_connect (window, "destroy", G_CALLBACK (gtk_main_quit), &window);
  gtk_widget_show_all (window);

  gtk_main ();
  return 0;
#endif
#if 1
  char *announcetext;
  DBusConnection *bus = NULL;
  DBusError error;
  DBusMessage *message = NULL;


  announcetext = argv[1];
  /*
   * Following codes for dbus can be referenced in <dbus/tools/dbus-send.c>
   */
  dbus_error_init (&error);


  bus = dbus_bus_get (DBUS_BUS_SESSION, &error);
  if (!bus)
  {
    g_warning ("Failed to connect to the D-BUS daemon: %s", error.message);
    dbus_error_free (&error);
    return FALSE;
  }

  if (dbus_error_is_set (&error))
  {
    g_warning ("Failed to acquire phone server service");
    dbus_error_free (&error);
    return FALSE;
  }

  message = dbus_message_new_signal (ANNOUNCE_PATH,
                                     ANNOUNCE_INTERFACE, "Notification");
  if (!message)
  {
    g_print ("Failed to contruct message call\n");
    dbus_error_free (&error);
    return -1;
  }

  dbus_int32_t V_INT32 = 2;     //gpe-applauncher/homescreen.h "NOTE_ALARM"
  const char *V_STRING = g_strdup (announcetext);

  if (V_STRING)
  {
    g_print ("V_STRING = %s\n", V_STRING);
    dbus_message_append_args (message, DBUS_TYPE_INT32, &V_INT32,
                              DBUS_TYPE_STRING, &V_STRING, DBUS_TYPE_INVALID);

    dbus_message_set_auto_start (message, TRUE);

    dbus_connection_send (bus, message, NULL);
    dbus_connection_flush (bus);

    dbus_message_unref (message);
    dbus_connection_close (bus);
  }


  g_print ("finishe\n");
  return 0;

#endif
}

/*
vi:ts=2:nowrap:ai:expandtab
*/

