/*	calendar - calendaring and scheduling application reference implementation
 *  basicframe.c - The main frame of calendar application,including soft key menu handling 
 *  and page switch handling.
 *
 *  Authors: YANG Lin <lin.yang@orange-ftgroup.com> 
 *  
 *  This software and associated documentation files (the "Software") 
 *  are copyright (C) 2005 LiPS Linux Phone Standards Forum [FranceTelecom] 
 *  All Rights Reserved. 
 *
 *  A copyright license is hereby granted for redistribution and use of 
 *  the Software in source and binary forms, with or without modification, 
 *  provided that the following conditions are met: 
 *  - Redistributions of source code must retain the above copyright notice, 
 *  this copyright license and the following disclaimer. 
 *  - Redistributions in binary form must reproduce the above copyright 
 *   notice, this copyright license and the following disclaimer in the 
 *  documentation and/or other materials provided with the distribution. 
 *  - Neither the name of LiPS  nor the names of its Members may be used 
 *  to endorse or promote products derived from the Software without 
 *  specific prior written permission. 
 *
 *  A patent license for any Necessary Claims owned by Members of LiPS Forum 
 *  to make, have made, use, import, offer to sell, lease and sell or otherwise 
 *  distribute any implementation compliant with the any specification adopted 
 *  by the LiPS Forumcan be obtained from the respective Members on reasonable 
 *  and non-discriminatory terms and conditions and under reciprocity, as 
 *  regulated in more detail in the Internal Policy of the LiPS Forum. 
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDER, ITS MEMBERS AND CONTRIBUTORS 
 *  "AS IS", AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
 *  THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE 
 *  AND NON-INFRINGEMENT ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER, 
 *  ITS MEMBERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
 *  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; 
 *  OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 *  WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 *  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
 *  POSSIBILITY OF SUCH DAMAGE. 
 */


#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <gtk/gtk.h>
#include <gpe/gpeskwindow.h>
#include <gdk/gdkkeysyms.h>


#include "basicframe.h"
#include "eventedit.h"
#include "monthview.h"
#include "weekview.h"
#include "dayview.h"
#include "setting.h"

static gint lastpage = 0;
static gint thispage = 0;

void
cal_set_page (guint page)
{
  lastpage = thispage;
  thispage = page;
}

guint
cal_get_lastpage (void)
{
  return lastpage;
}


void
switch_page_cb (GtkNotebook *notebook,
                GtkNotebookPage *page, guint page_num, gpointer user_data)
{
  GtkAction *action;
  GList *actionGroup;
  GList *actions;
  guint i = 0, length;

  actionGroup = gtk_ui_manager_get_action_groups (self.ui_manager);
  actions =
    gtk_action_group_list_actions (GTK_ACTION_GROUP (actionGroup->data));


  length = g_list_length (actions);

  while (i < length)
  {
    g_object_set (GTK_ACTION (actions->data), "visible", TRUE, NULL);
    actions = actions->next;
    i++;
  }


  switch (page_num)
  {
  case MONTH_VIEW_PAGE:

    action =
      gtk_ui_manager_get_action (self.ui_manager, "/ui/popup-left/monthview");
    g_object_set (action, "visible", FALSE, NULL);

    action =
      gtk_ui_manager_get_action (self.ui_manager, "/ui/popup-left/deleteone");
    g_object_set (action, "visible", FALSE, NULL);

    gpe_skwindow_set_softkey_menu (GPE_SKWINDOW (self.window), SK_LEFT,
                                   self.menu_left);
    gpe_skwindow_set_softkey_menu (GPE_SKWINDOW (self.window), SK_RIGHT, NULL);

    gpe_skwindow_set_softkey_label (GPE_SKWINDOW (self.window), SK_LEFT,
                                    "Options");
    gpe_skwindow_set_softkey_label (GPE_SKWINDOW (self.window), SK_RIGHT,
                                    "Back");


    cal_set_page (MONTH_VIEW_PAGE);
    break;
  case DAY_VIEW_PAGE:

    action =
      gtk_ui_manager_get_action (self.ui_manager, "/ui/popup-left/dayview");
    g_object_set (action, "visible", FALSE, NULL);

    action =
      gtk_ui_manager_get_action (self.ui_manager,
                                 "/ui/popup-left/deleteevent/deleteall");
    g_object_set (action, "visible", FALSE, NULL);

    action =
      gtk_ui_manager_get_action (self.ui_manager,
                                 "/ui/popup-left/deleteevent/deletebeforedate");
    g_object_set (action, "visible", FALSE, NULL);


    gpe_skwindow_set_softkey_menu (GPE_SKWINDOW (self.window), SK_LEFT,
                                   self.menu_left);
    gpe_skwindow_set_softkey_menu (GPE_SKWINDOW (self.window), SK_RIGHT, NULL);

    gpe_skwindow_set_softkey_label (GPE_SKWINDOW (self.window), SK_LEFT,
                                    "Options");
    gpe_skwindow_set_softkey_label (GPE_SKWINDOW (self.window), SK_RIGHT,
                                    "Back");


    cal_set_page (DAY_VIEW_PAGE);

    break;
  case WEEK_VIEW_PAGE:
    action =
      gtk_ui_manager_get_action (self.ui_manager, "/ui/popup-left/weekview");
    g_object_set (action, "visible", FALSE, NULL);
    action =
      gtk_ui_manager_get_action (self.ui_manager, "/ui/popup-left/deleteevent");
    g_object_set (action, "visible", FALSE, NULL);

    action =
      gtk_ui_manager_get_action (self.ui_manager, "/ui/popup-left/deleteone");
    g_object_set (action, "visible", FALSE, NULL);

    gpe_skwindow_set_softkey_menu (GPE_SKWINDOW (self.window), SK_LEFT,
                                   self.menu_left);
    gpe_skwindow_set_softkey_menu (GPE_SKWINDOW (self.window), SK_RIGHT, NULL);
    gpe_skwindow_set_softkey_label (GPE_SKWINDOW (self.window), SK_LEFT,
                                    "Options");
    gpe_skwindow_set_softkey_label (GPE_SKWINDOW (self.window), SK_RIGHT,
                                    "Back");


    cal_set_page (WEEK_VIEW_PAGE);
    break;
  case EVENT_EDIT_PAGE:
    gpe_skwindow_set_softkey_menu (GPE_SKWINDOW (self.window), SK_LEFT, NULL);
    gpe_skwindow_set_softkey_menu (GPE_SKWINDOW (self.window), SK_RIGHT, NULL);
    gpe_skwindow_set_softkey_label (GPE_SKWINDOW (self.window), SK_LEFT,
                                    "Save");
    gpe_skwindow_set_softkey_label (GPE_SKWINDOW (self.window), SK_RIGHT,
                                    "Back");


    cal_set_page (EVENT_EDIT_PAGE);
    break;
  case SETTING_PAGE:
    gpe_skwindow_set_softkey_menu (GPE_SKWINDOW (self.window), SK_LEFT, NULL);
    gpe_skwindow_set_softkey_menu (GPE_SKWINDOW (self.window), SK_RIGHT, NULL);
    gpe_skwindow_set_softkey_label (GPE_SKWINDOW (self.window), SK_LEFT,
                                    "Save");
    gpe_skwindow_set_softkey_label (GPE_SKWINDOW (self.window), SK_RIGHT,
                                    "Back");


    cal_set_page (SETTING_PAGE);
    break;
  default:
    break;
  }
}

static gboolean
save_event (GtkWidget *window)
{
  GtkWidget *view;

  view =
    gtk_notebook_get_nth_page (GTK_NOTEBOOK (self.notebook), EVENT_EDIT_PAGE);
  if (CAL_IS_EVENT_EDIT (view))
    return cal_save_event (CAL_EVENT_EDIT (view));
}

static gboolean
save_setting (GtkWidget *window)
{
  GtkWidget *view;

  view = gtk_notebook_get_nth_page (GTK_NOTEBOOK (self.notebook), SETTING_PAGE);
  if (CAL_IS_SETTING_PAGE (view))
    return cal_save_setting (CAL_SETTING_PAGE (view));
}

void
softkey_activated_cb (GtkWidget *widget, gint key_number)
{
  GtkWidget *window = widget, *temp;
  const gchar *label = NULL;
  gint page_num;


  page_num = gtk_notebook_get_current_page (GTK_NOTEBOOK (self.notebook));
  if (key_number == SK_LEFT)
  {
    switch (page_num)
    {
    case MONTH_VIEW_PAGE:
    case DAY_VIEW_PAGE:
    case WEEK_VIEW_PAGE:
      label = gpe_skwindow_get_softkey_label (GPE_SKWINDOW (window), SK_RIGHT);
      temp = gpe_skwindow_get_softkey_menu (GPE_SKWINDOW (window), SK_LEFT);
      if ((label != "Cancel") && (temp != NULL))
      {
        gpe_skwindow_set_softkey_label (GPE_SKWINDOW (window), SK_RIGHT,
                                        "Cancel");
      }
      break;
    case EVENT_EDIT_PAGE:
      if (save_event (window))
      {
        update_all_views ();
        gtk_notebook_set_current_page (GTK_NOTEBOOK (self.notebook),
                                       cal_get_lastpage ());
      }

      break;
    case SETTING_PAGE:
      if (save_setting (window))
      {
        update_all_views ();
        gtk_notebook_set_current_page (GTK_NOTEBOOK (self.notebook),
                                       cal_get_lastpage ());
      }

      break;
    }
  }

  if (key_number == SK_RIGHT)
  {
    switch (page_num)
    {
    case MONTH_VIEW_PAGE:
      gtk_main_quit ();
      break;
    case DAY_VIEW_PAGE:
      gtk_notebook_set_current_page (GTK_NOTEBOOK (self.notebook),
                                     MONTH_VIEW_PAGE);
      break;
    case WEEK_VIEW_PAGE:
      gtk_notebook_set_current_page (GTK_NOTEBOOK (self.notebook),
                                     MONTH_VIEW_PAGE);
      break;
    case EVENT_EDIT_PAGE:

      gtk_notebook_set_current_page (GTK_NOTEBOOK (self.notebook),
                                     cal_get_lastpage ());
      break;
    case SETTING_PAGE:

      gtk_notebook_set_current_page (GTK_NOTEBOOK (self.notebook),
                                     cal_get_lastpage ());
      break;
    default:
      break;
    }
  }
}

gboolean
popup_key_press_cb (GtkWidget *widget, GdkEventKey *event, gpointer user_data)
{
  GtkWidget *window = self.window;
  GtkWidget *notebook;
  const gchar *label = NULL;
  label = gpe_skwindow_get_softkey_label (GPE_SKWINDOW (window), SK_RIGHT);
  if (event->keyval == GDK_F4 || event->keyval == GDK_KP_F4)
  {
    gtk_menu_popdown (GTK_MENU (self.menu_left));
    gpe_skwindow_set_softkey_label (GPE_SKWINDOW (window), SK_RIGHT, "Back");
    notebook = self.notebook;
    gtk_grab_add (GTK_WIDGET (notebook));


    return TRUE;
  }
  return FALSE;
}

/*
vi:ts=2:nowrap:ai:expandtab
*/

