/*	calendar - calendaring and scheduling application reference implementation
 *  calenabler.c - The interface to render calendar enabler.
 *  Authors: YANG Lin <lin.yang@orange-ftgroup.com> 
 *  
 *  This software and associated documentation files (the "Software") 
 *  are copyright (C) 2005 LiPS Linux Phone Standards Forum [FranceTelecom] 
 *  All Rights Reserved. 
 *
 *  A copyright license is hereby granted for redistribution and use of 
 *  the Software in source and binary forms, with or without modification, 
 *  provided that the following conditions are met: 
 *  - Redistributions of source code must retain the above copyright notice, 
 *  this copyright license and the following disclaimer. 
 *  - Redistributions in binary form must reproduce the above copyright 
 *   notice, this copyright license and the following disclaimer in the 
 *  documentation and/or other materials provided with the distribution. 
 *  - Neither the name of LiPS  nor the names of its Members may be used 
 *  to endorse or promote products derived from the Software without 
 *  specific prior written permission. 
 *
 *  A patent license for any Necessary Claims owned by Members of LiPS Forum 
 *  to make, have made, use, import, offer to sell, lease and sell or otherwise 
 *  distribute any implementation compliant with the any specification adopted 
 *  by the LiPS Forumcan be obtained from the respective Members on reasonable 
 *  and non-discriminatory terms and conditions and under reciprocity, as 
 *  regulated in more detail in the Internal Policy of the LiPS Forum. 
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDER, ITS MEMBERS AND CONTRIBUTORS 
 *  "AS IS", AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
 *  THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE 
 *  AND NON-INFRINGEMENT ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER, 
 *  ITS MEMBERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
 *  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; 
 *  OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 *  WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 *  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
 *  POSSIBILITY OF SUCH DAMAGE. 
 */

#include <stdio.h>

#include <glib.h>
#include <gtk/gtk.h>

#include <calenabler/cal.h>

static cal_eventbook_t *book;

gboolean
cal_enabler_init (void)
{
  guint ev_id = 0;

  book = eventbook_connect (ev_id);

  if (!book)
    return FALSE;

  return TRUE;
}

void
cal_enabler_destroy (void)
{
  eventbook_disconnect (book);

  return;
}

cal_eventbook_t *
cal_enabler_get_book (void)
{
  return book;
}

char *
uid2str (unsigned char *uid)
{
  gchar *str;

  str =
    g_strdup_printf
    ("[%02x %02x %02x %02x %02x %02x %02x %02x %02x %02x %02x %02x %02x %02x %02x %02x]",
     uid[0], uid[1], uid[2], uid[3], uid[4], uid[5], uid[6], uid[7], uid[8],
     uid[9], uid[10], uid[11], uid[12], uid[13], uid[14], uid[15]);

  return str;
}

void
cal_print_uid (const unsigned char *uid)
{
  gchar *tempstr = NULL;

  //g_print ("0x%08d\n", (guint32) uid);
  tempstr =
    g_strdup_printf
    ("[%02x %02x %02x %02x %02x %02x %02x %02x %02x %02x %02x %02x %02x %02x %02x %02x]",
     uid[0], uid[1], uid[2], uid[3], uid[4], uid[5], uid[6], uid[7], uid[8],
     uid[9], uid[10], uid[11], uid[12], uid[13], uid[14], uid[15]);

  g_print ("%s:uid:%-20s\n", __FUNCTION__, tempstr);

  g_free (tempstr);

}

void
cal_print_event (const cal_event_t *event)
{
  gchar *uid = NULL;
  gchar *uid_str = NULL;
  gchar *tmp_str = NULL;
  cal_time_t tmp_time;

  g_print ("-cal_event_t[0x%08x]:\n", (unsigned int) event);

  uid = event_get_uid (event);
  if (uid != NULL)
  {
    uid_str = uid2str (uid);
    g_free (uid);
  }
  tmp_str = event_get_description (event);
  g_print ("|---> UID:         %-20s\n", uid_str ? uid_str : "NULL");
  g_print ("|---> DESCRIPTION: %-20s\n", tmp_str ? tmp_str : "NULL");
  g_free (tmp_str);
  tmp_time = event_get_start (event);
  if (tmp_time)
    g_print ("|---> START:       %s\n", ctime (&tmp_time));
  tmp_time = event_get_end (event);
  if (tmp_time)
    g_print ("|---> END:       %s\n", ctime (&tmp_time));

  g_print ("|---> HAS_ALARM:       %d\n", event_get_has_alarm (event));
  g_print ("|---> ALARM_DELAY:       %d\n", event_get_alarm_delay (event));
  /*g_print ("|---> REPEAT TYPE:     %d\n", event_get_repeat_type (event));
     if (!event_get_repeat_forever (event))
     {
     tmp_time = event_get_repeat_end (event);  
     g_print ("|---> REPEAT END:     %s\n", ctime(&tmp_time));

     } */

  g_free (uid_str);

  return;
}

void
cal_event_list_print (GSList *list)
{
  cal_event_t *ev;
  GSList *iter;


  for (iter = list; iter != NULL; iter = iter->next)
  {
    ev = iter->data;
#ifdef DEBUG
    cal_print_event (ev);
#endif
  }
}

void
cal_event_list_destroy (GSList *list)
{
  GSList *iter;

  for (iter = list; iter != NULL; iter = iter->next)
  {
    cal_event_t *ev = iter->data;
    event_free (ev);
  }
  g_slist_free (list);
  list = NULL;

}

static gint
day_events_sort_func (const cal_event_t *ev1, const cal_event_t *ev2)
{
  time_t start1, start2;
  struct tm tm1, tm2;



  start1 = event_get_start (ev1);
  start2 = event_get_start (ev2);

  localtime_r (&start1, &tm1);
  localtime_r (&start2, &tm2);

  if ((tm1.tm_hour > tm2.tm_hour) 
      ||((tm1.tm_hour == tm2.tm_hour) 
      && (tm1.tm_min >= tm2.tm_min)))
    return 1;
  else
    return 0;
}

GSList *
cal_event_list_init (time_t from, time_t to)
{
  GSList *list = NULL;
  cal_event_iter_t *eviter = event_iterator_new ();
  cal_eventbook_t *book;
  guint i;

  book = (cal_eventbook_t *) cal_enabler_get_book ();

  eventbook_get_events_for_period (book, eviter, from, to);
  for (i = 0; i < event_iterator_size (eviter); i++)
  {
    cal_event_t *ev;

    ev = event_iterator_current (eviter);
    /*print_event(ev);*/
    event_iterator_next (eviter);
    /*list = g_slist_append (list, ev);  */
    list =
      g_slist_insert_sorted (list, ev, (GCompareFunc) day_events_sort_func);
  }

  return list;
}

/*
vi:ts=2:nowrap:ai:expandtab
*/

