/*	calendar - calendaring and scheduling application reference implementation
 *  dayview.c - day view page
 *
 *  Authors: YANG Lin <lin.yang@orange-ftgroup.com> 
 *  
 *  This software and associated documentation files (the "Software") 
 *  are copyright (C) 2005 LiPS Linux Phone Standards Forum [FranceTelecom] 
 *  All Rights Reserved. 
 *
 *  A copyright license is hereby granted for redistribution and use of 
 *  the Software in source and binary forms, with or without modification, 
 *  provided that the following conditions are met: 
 *  - Redistributions of source code must retain the above copyright notice, 
 *  this copyright license and the following disclaimer. 
 *  - Redistributions in binary form must reproduce the above copyright 
 *   notice, this copyright license and the following disclaimer in the 
 *  documentation and/or other materials provided with the distribution. 
 *  - Neither the name of LiPS  nor the names of its Members may be used 
 *  to endorse or promote products derived from the Software without 
 *  specific prior written permission. 
 *
 *  A patent license for any Necessary Claims owned by Members of LiPS Forum 
 *  to make, have made, use, import, offer to sell, lease and sell or otherwise 
 *  distribute any implementation compliant with the any specification adopted 
 *  by the LiPS Forumcan be obtained from the respective Members on reasonable 
 *  and non-discriminatory terms and conditions and under reciprocity, as 
 *  regulated in more detail in the Internal Policy of the LiPS Forum. 
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDER, ITS MEMBERS AND CONTRIBUTORS 
 *  "AS IS", AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
 *  THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE 
 *  AND NON-INFRINGEMENT ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER, 
 *  ITS MEMBERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
 *  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; 
 *  OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 *  WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 *  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
 *  POSSIBILITY OF SUCH DAMAGE. 
 */

#include <sys/types.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <libintl.h>
#include <locale.h>

#include <gtk/gtk.h>
#include <glib.h>

#include <gdk/gdkkeysyms.h>
#include <calenabler/cal.h>
#include <gpe/gpeskwindow.h>
#include <gemwidget/gem.h>

#include "timemanage.h"
#include "calenabler.h"
#include "dayview.h"
#include "basicframe.h"


static void cal_day_view_class_init (CalDayViewClass *klass);
static void cal_day_view_init (CalDayView *view);

enum
{
  COL_TIME = 0,
  COL_SUM,
  COL_PIC,
  COL_NTH,
  NUM_COLS
};

GType
cal_day_view_get_type (void)
{
  static GType dayview_type = 0;

  if (!dayview_type)
  {
    static const GTypeInfo dayview_info = {
      sizeof (CalDayViewClass),
      NULL,                     /* base_init */
      NULL,                     /* base_finalize */
      (GClassInitFunc) cal_day_view_class_init,
      NULL,                     /* class_finalize */
      NULL,                     /* class_data */
      sizeof (CalDayView),
      0,                        /* n_preallocs */
      (GInstanceInitFunc) cal_day_view_init,
    };

    dayview_type =
      g_type_register_static (GTK_TYPE_FRAME, "CalDayView", &dayview_info, 0);
  }

  return dayview_type;
}

static void
cal_day_view_class_init (CalDayViewClass *klass)
{
  return;
}

static GtkWidget *
create_treeview_dayevents (void)
{
  GtkWidget *treeview;
  GtkListStore *liststore;
  GtkCellRenderer *renderer;
  GtkTreeViewColumn *col;
  GtkTreeSelection *treesel;
  GtkTreePath *path;

  liststore = gtk_list_store_new (NUM_COLS, G_TYPE_STRING, G_TYPE_STRING,
                                  GDK_TYPE_PIXBUF, G_TYPE_INT);

  treeview = gtk_tree_view_new_with_model (GTK_TREE_MODEL (liststore));

  gtk_tree_view_set_headers_visible (GTK_TREE_VIEW (treeview), FALSE);
  treesel = gtk_tree_view_get_selection (GTK_TREE_VIEW (treeview));
  gtk_tree_selection_set_mode (treesel, GTK_SELECTION_BROWSE);

/*Add columns to treeview*/
  renderer = gtk_cell_renderer_text_new ();
  gtk_tree_view_insert_column_with_attributes (GTK_TREE_VIEW (treeview),
                                               -1, "Time", renderer, "text",
                                               COL_TIME, NULL);
  col = gtk_tree_view_column_new ();
  renderer = gtk_cell_renderer_text_new ();
  gtk_tree_view_insert_column_with_attributes (GTK_TREE_VIEW (treeview), -1,
                                               "Summary", renderer, "text",
                                               COL_SUM, NULL);
  gtk_cell_renderer_set_fixed_size (renderer, 80, -1);
  col = gtk_tree_view_column_new ();
  gtk_tree_view_column_set_title (col, "Picutre");
  gtk_tree_view_append_column (GTK_TREE_VIEW (treeview), col);
  renderer = gtk_cell_renderer_pixbuf_new ();
  /*gtk_cell_renderer_set_fixed_size(renderer, 30, 30); */
  gtk_tree_view_column_pack_start (col, renderer, TRUE);
  gtk_tree_view_column_add_attribute (col, renderer, "pixbuf", COL_PIC);

/*Set the cursor position*/
  path = gtk_tree_path_new_first ();
  gtk_tree_view_set_cursor (GTK_TREE_VIEW (treeview), path, NULL, FALSE);
  gtk_tree_path_free (path);


  g_object_unref (liststore);

  return treeview;
}

enum
{
  COL_DES = 0,
  COL_N,
  NUM_COLUMS
};

static GtkWidget *
create_treeview_delete_reev_warning (void)
{
  GtkWidget *treeview;
  GtkListStore *liststore;
  GtkCellRenderer *renderer;

  GtkTreeSelection *treesel;
  GtkTreePath *path;
  GtkTreeIter titer;


  liststore = gtk_list_store_new (NUM_COLUMS, G_TYPE_STRING, G_TYPE_INT);

  /*list_store = create_items_model ();*/
  treeview = gtk_tree_view_new_with_model (GTK_TREE_MODEL (liststore));

  gtk_tree_view_set_headers_visible (GTK_TREE_VIEW (treeview), FALSE);
  treesel = gtk_tree_view_get_selection (GTK_TREE_VIEW (treeview));
  gtk_tree_selection_set_mode (treesel, GTK_SELECTION_BROWSE);

/*Add columns to treeview*/
  renderer = gtk_cell_renderer_text_new ();
  gtk_tree_view_insert_column_with_attributes (GTK_TREE_VIEW (treeview),
                                               -1, "Description", renderer,
                                               "text", COL_DES, NULL);

/*Add rows*/
  gtk_list_store_append (liststore, &titer);
  gtk_list_store_set (liststore, &titer, COL_DES, "Delete all occurrences?",
                      COL_N, 0, -1);
  gtk_list_store_append (liststore, &titer);
  gtk_list_store_set (liststore, &titer, COL_DES,
                      "Delete only this occurrence?", COL_N, 1, -1);

/*Set the cursor position*/
  path = gtk_tree_path_new_first ();
  gtk_tree_view_set_cursor (GTK_TREE_VIEW (treeview), path, NULL, FALSE);
  gtk_tree_path_free (path);


  g_object_unref (liststore);

  return treeview;
}

void
day_view_style_init (CalDayView *view)
{
  guint widget_width, i;
  static gboolean flag = FALSE;

  widget_width = view->treeview_dayevents->allocation.width;

  if (!flag)
  {
    view->light_color.red = 60000;
    view->light_color.green = 60000;
    view->light_color.blue = 60000;

    view->time_color.red = 20000;
    view->time_color.green = 20000;
    view->time_color.blue = 20000;

    view->dark_color.red = 45000;
    view->dark_color.green = 45000;
    view->dark_color.blue = 45000;

    view->light_style =
      gtk_style_copy (gtk_widget_get_style (view->treeview_dayevents));
    view->dark_style =
      gtk_style_copy (gtk_widget_get_style (view->treeview_dayevents));
    view->time_style =
      gtk_style_copy (gtk_widget_get_style (view->treeview_dayevents));

    for (i = 0; i < 5; i++)
    {
      view->light_style->base[i] = view->light_color;
      view->dark_style->base[i] = view->dark_color;
      view->time_style->base[i] = view->time_color;
      view->time_style->fg[i] = view->light_color;
    }
    flag = TRUE;
  }
}


void
day_view_day_events_update (CalDayView *view)
{
  time_t viewtime = cal_get_viewtime ();
  struct tm tm_start, tm_end;
  time_t start, end;


  g_assert (CAL_IS_DAY_VIEW (view));

  localtime_r (&viewtime, &tm_start);
  localtime_r (&viewtime, &tm_end);


  tm_start.tm_hour = 0;
  tm_start.tm_min = 0;
  tm_start.tm_sec = 0;

  start = mktime (&tm_start);
  localtime_r (&viewtime, &tm_end);
  tm_end.tm_hour = 23;
  tm_end.tm_min = 59;
  tm_end.tm_sec = 59;
  end = mktime (&tm_end);


  if (view->day_events)
    cal_event_list_destroy (view->day_events);

  view->day_events = cal_event_list_init (start, end);

  cal_event_list_print (view->day_events);

}

gboolean
day_view_day_indicate_update (CalDayView *view)
{
  gchar buffer[255];
  struct tm tm;
  time_t viewtime = cal_get_viewtime ();

  localtime_r (&viewtime, &tm);
  strftime (buffer, sizeof (buffer), DAY_TIME_FMT, &tm);
  gtk_label_set_text (GTK_LABEL (view->date_indicate), buffer);

  return TRUE;
}

gboolean
cal_day_view_update (CalDayView *view)
{
  GtkTreeIter titer;
  GtkTreeModel *model =
    gtk_tree_view_get_model (GTK_TREE_VIEW (view->treeview_dayevents));
  GtkTreePath *path;
  GtkAction *action;

  action =
    gtk_ui_manager_get_action (self.ui_manager, "/ui/popup-left/deleteone");
#ifdef DEBUG
  printf ("%s:\n", __FUNCTION__);
#endif
  gtk_list_store_clear (GTK_LIST_STORE (model));

  day_view_day_events_update (view);

  if (!view->day_events)
  {

    gtk_list_store_append (GTK_LIST_STORE (model), &titer);
    gtk_list_store_set (GTK_LIST_STORE (model), &titer,
                        COL_TIME, "No events!", -1);

    g_object_set (action, "sensitive", FALSE, NULL);
  }
  else
  {
    GSList *iter;
    cal_event_t *ev = NULL;
    char timebuf1[10], timebuf2[10], timebuf[30];
    gint nth = 0;


    g_object_set (action, "sensitive", TRUE, NULL);

    for (iter = view->day_events; iter != NULL; iter = iter->next)
    {
      time_t starttime, endtime;
      struct tm tm;
      gchar *description = NULL;
      gchar *picpath = NULL;
      gint rt;
      gchar *res_path = NULL;
      GdkPixbuf *pic;
      GError *error = NULL;

      ev = iter->data;


      description = event_get_description (ev);


      starttime = event_get_start (ev);
      endtime = event_get_end (ev);

      localtime_r (&starttime, &tm);
      strftime (timebuf1, sizeof (timebuf1), TIMEFMT, &tm);

      localtime_r (&endtime, &tm);
      strftime (timebuf2, sizeof (timebuf2), TIMEFMT, &tm);

      sprintf (timebuf, "%s-%s", timebuf1, timebuf2);

      res_path = getenv ("CAL_RES_PATH");
      rt = event_get_repeat_type (ev);
      if (event_get_has_alarm (ev))
        if (rt != REPEAT_NONE)
          picpath = g_build_filename (res_path, "pixmaps/bell_recur.png", NULL);
        else
          picpath = g_build_filename (res_path, "pixmaps/bell.png", NULL);
      else if (rt != REPEAT_NONE)
        picpath = g_build_filename (res_path, "pixmaps/recur.png", NULL);
      else
        picpath = g_build_filename (res_path, "pixmaps/nothing.png", NULL);
      printf ("%s picpath =%s\n", __FUNCTION__, picpath);
      pic = gdk_pixbuf_new_from_file (picpath, &error);

      gtk_list_store_append (GTK_LIST_STORE (model), &titer);
      gtk_list_store_set (GTK_LIST_STORE (model), &titer, COL_TIME, timebuf,
                          COL_SUM, description, COL_PIC, pic, COL_NTH, nth, -1);

      nth++;

      g_free (description);
      g_free (picpath);

    }

  }
  day_view_day_indicate_update (view);

  /*Set the cursor to the first line */
  path = gtk_tree_path_new_first ();
  gtk_tree_view_set_cursor (GTK_TREE_VIEW (view->treeview_dayevents), path,
                            NULL, FALSE);
  gtk_tree_path_free (path);


  return TRUE;
}



static gboolean
_cal_day_view_delete_one_event (CalDayView *view, gint flag)
{
  GtkTreeModel *model = NULL;
  GtkTreeIter iter;
  GtkTreePath *path = NULL;
  cal_eventbook_t *book = cal_enabler_get_book ();


#ifdef DEBUG
  printf ("%s:flag = %d\n", __FUNCTION__, flag);
#endif


  gtk_tree_view_get_cursor (GTK_TREE_VIEW (view->treeview_dayevents), &path,
                            NULL);
  model = gtk_tree_view_get_model (GTK_TREE_VIEW (view->treeview_dayevents));

  if (path)
  {
    if (gtk_tree_model_get_iter (model, &iter, path))
    {

      cal_event_t *ev;
      gint nth;
      gint ret;


      gtk_tree_model_get (model, &iter, COL_NTH, &nth, -1);

      ev = (cal_event_t *) g_slist_nth_data (view->day_events, nth);

      if (flag == 0)            /*remove one event*/
      {
        gchar *uidinfo = NULL;
#ifdef DEBUG
        printf ("%s: remove event\n", __FUNCTION__);
#endif
        uidinfo = event_get_uid (ev);
        cal_print_uid (uidinfo);
        ret = eventbook_remove_event (book, uidinfo);

        g_free (uidinfo);
      }
      else if (flag == 1)       /*add exception*/
      {
        time_t exception, starttime, viewtime = cal_get_viewtime ();
        struct tm starttm, viewtm;


        starttime = event_get_start (ev);
        localtime_r (&starttime, &starttm);
        localtime_r (&viewtime, &viewtm);
        viewtm.tm_sec = starttm.tm_sec;
        viewtm.tm_min = starttm.tm_min;
        viewtm.tm_hour = starttm.tm_hour;
        exception = mktime (&viewtm);


        event_set_has_exception (ev, TRUE);
        event_add_exception (ev, exception);

        eventbook_update_event (book, ev);
      }
    }
  }
  return TRUE;
}


void
delete_one_softkey_activated_cb (GtkWidget *widget, gint key_number)
{
  GtkNotebook *notebook;
  GtkWidget *view;


  if (key_number == SK_LEFT)
  {

    notebook = (GtkNotebook *) self.notebook;
    view = gtk_notebook_get_nth_page (GTK_NOTEBOOK (notebook), DAY_VIEW_PAGE);
    if (CAL_IS_DAY_VIEW (view))
      if (_cal_day_view_delete_one_event (CAL_DAY_VIEW (view), 0))
        update_all_views ();
  }
  gtk_widget_destroy (widget);
}

void
delete_reev_softkey_activated_cb (GtkWidget *widget, gint key_number)
{
  GtkWidget *treeview;
  GtkTreeModel *model = NULL;
  GtkTreeIter iter;
  GtkTreePath *path = NULL;
  guint nth;

  if (key_number == SK_LEFT)
  {
    GtkNotebook *notebook;
    GtkWidget *view;

    treeview = g_object_get_data (G_OBJECT (widget), "tree_view");


    gtk_tree_view_get_cursor (GTK_TREE_VIEW (treeview), &path, NULL);
    model = gtk_tree_view_get_model (GTK_TREE_VIEW (treeview));
    gtk_tree_model_get_iter (model, &iter, path);
    gtk_tree_model_get (model, &iter, COL_N, &nth, -1);


    notebook = (GtkNotebook *) self.notebook;
    view = gtk_notebook_get_nth_page (GTK_NOTEBOOK (notebook), DAY_VIEW_PAGE);
    if (CAL_IS_DAY_VIEW (view))
    {
      if (nth == 0)
        _cal_day_view_delete_one_event (CAL_DAY_VIEW (view), 0);
      else
        _cal_day_view_delete_one_event (CAL_DAY_VIEW (view), 1);

      update_all_views ();
    }

  }
  gtk_widget_destroy (widget);
}

static gboolean
reev_warning_key_press_event_cb (GtkWidget *widget, GdkEventKey *k,
                                 GtkWidget *user_data)
{
  CalDayView *view = (CalDayView *) user_data;
  GtkTreeModel *model = NULL;
  GtkTreeIter iter;
  GtkTreePath *path = NULL;
  guint nth;
  GtkWidget *dialog = g_object_get_data (G_OBJECT (view), "warning_dialog");

  g_print ("enter %s\n", __FUNCTION__);
  gtk_tree_view_get_cursor (GTK_TREE_VIEW (widget), &path, NULL);
  model = gtk_tree_view_get_model (GTK_TREE_VIEW (widget));
  gtk_tree_model_get_iter (model, &iter, path);
  gtk_tree_model_get (model, &iter, COL_N, &nth, -1);

  switch (k->keyval)
  {
  case GDK_Return:
    g_signal_emit_by_name (G_OBJECT (dialog), "softkey_activated", SK_LEFT);
    break;

  case GDK_Down:
  case GDK_Up:
    if (path)
    {

      if (nth == 1)
        gtk_tree_path_prev (path);
      else
        gtk_tree_path_next (path);

      gtk_tree_view_set_cursor (GTK_TREE_VIEW (widget), path, NULL, FALSE);
    }

    break;

  }

  gtk_tree_path_free (path);
  return TRUE;
}

void
cal_day_view_delete_one_event ()
{
  GtkNotebook *notebook;
  GtkWidget *view;

  GtkTreeModel *model = NULL;
  GtkTreeIter iter;
  GtkTreePath *path = NULL;
  cal_eventbook_t *book = cal_enabler_get_book ();


  notebook = (GtkNotebook *) self.notebook;
  view = gtk_notebook_get_nth_page (GTK_NOTEBOOK (notebook), DAY_VIEW_PAGE);

  gtk_tree_view_get_cursor (GTK_TREE_VIEW
                            (CAL_DAY_VIEW (view)->treeview_dayevents), &path,
                            NULL);
  model =
    gtk_tree_view_get_model (GTK_TREE_VIEW
                             (CAL_DAY_VIEW (view)->treeview_dayevents));

  if (path)
  {
    if (gtk_tree_model_get_iter (model, &iter, path))
    {
      gchar *uidinfo = NULL;
      cal_event_t *ev;
      gint nth;

      gtk_tree_model_get (model, &iter, COL_NTH, &nth, -1);
#ifdef DEBUG
      printf ("%s:nth=%d\n", __FUNCTION__, nth);
#endif
      ev =
        (cal_event_t *) g_slist_nth_data (CAL_DAY_VIEW (view)->day_events, nth);

      cal_print_event (ev);
      if (event_get_repeat_type (ev) == REPEAT_NONE)
      {
        GtkWidget *dialog;

        dialog = gpe_skwindow_new ();

        gem_sk_window_set_delete_warning (GPE_SKWINDOW (dialog),
                                          "Delete event?");
        gem_sk_dialog_set_position (GTK_WINDOW (self.window),
                                    GPE_SKWINDOW (dialog), 0);

        g_signal_connect_swapped (dialog, "softkey_activated",
                                  G_CALLBACK (delete_one_softkey_activated_cb),
                                  dialog);
        gtk_widget_show_all (dialog);
      }
      else
      {
        GtkWidget *dialog;
        GtkWidget *frame;
        GtkWidget *treeview;

        GdkColor blue;

        blue.red = 53199;
        blue.green = 59110;
        blue.blue = 65535;

        dialog = gpe_skwindow_new ();
        g_object_set_data (G_OBJECT (view), "warning_dialog", dialog);
        gem_sk_window_set_dialog (GPE_SKWINDOW (dialog));
        gtk_widget_modify_bg (GTK_WIDGET (dialog), GTK_STATE_NORMAL, &blue);
        gtk_container_set_border_width (GTK_CONTAINER (dialog), 2);


        frame = gtk_frame_new (NULL);
        gtk_container_add (GTK_CONTAINER (dialog), frame);

        treeview = create_treeview_delete_reev_warning ();
        gtk_container_add (GTK_CONTAINER (frame), treeview);
        gtk_widget_modify_bg (GTK_WIDGET (treeview), GTK_STATE_NORMAL, &blue);
        g_object_set_data (G_OBJECT (dialog), "tree_view", treeview);

        g_signal_connect (G_OBJECT (treeview), "key_press_event",
                          G_CALLBACK (reev_warning_key_press_event_cb), view);



        gem_sk_dialog_set_position (GTK_WINDOW (self.window),
                                    GPE_SKWINDOW (dialog), 30);

        g_signal_connect_swapped (dialog, "softkey_activated",
                                  G_CALLBACK (delete_reev_softkey_activated_cb),
                                  dialog);
        gtk_widget_show_all (dialog);
      }
      g_free (uidinfo);

    }
  }
}

gboolean
cal_day_view_delete_day_events (CalDayView *view)
{
  cal_eventbook_t *book = cal_enabler_get_book ();
  gboolean flag = FALSE;
  GSList *iter;

  for (iter = view->day_events; iter != NULL; iter = iter->next)
  {
    cal_event_t *ev = iter->data;
    gchar *uid;
    gint ret;

    flag = TRUE;

    uid = event_get_uid (ev);
    ret = eventbook_remove_event (book, uid);

    g_free (uid);

  }

  return flag;
}

static gboolean
day_view_button_press_cb (GtkWidget *widget,
                          GdkEventButton *event, gpointer d)
{
  if (event->type == GDK_BUTTON_PRESS)
  {

  }
}
static gboolean
day_view_key_press_event_cb (GtkWidget *widget, GdkEventKey *k,
                             GtkWidget *user_data)
{
  GtkTreeModel *model = NULL;
  GtkTreeIter iter;
  CalDayView *view = (CalDayView *) user_data;
  GtkTreePath *path = NULL;

  /*g_print ("enter %s\n", __FUNCTION__);*/
  gtk_tree_view_get_cursor (GTK_TREE_VIEW (widget), &path, NULL);
  model = gtk_tree_view_get_model (GTK_TREE_VIEW (widget));

  switch (k->keyval)
  {
  case GDK_Return:
    if (path)
    {
      if (gtk_tree_model_get_iter (model, &iter, path))
      {
        gchar *time;
        guint nth;
        time_t viewtime = cal_get_viewtime ();
        struct tm tm;

        localtime_r (&viewtime, &tm);

        gtk_tree_model_get (model, &iter, COL_TIME, &time, COL_NTH, &nth, -1);


        if (view->day_events)
        {
          cal_event_t *ev;

          strptime (time, TIMEFMT, &tm);
          viewtime = mktime (&tm);
          cal_set_viewtime (viewtime);

          ev = (cal_event_t *) g_slist_nth_data (view->day_events, nth);
          //cal_print_event(ev);
          switch_to_edit_event (ev);
        }


        g_free (time);
      }
    }
    break;

  case GDK_Down:
    {
      if (path)
      {
        GtkTreePath *p;
        gint count;

        count = gtk_tree_model_iter_n_children (model, NULL);
        p = gtk_tree_path_new_from_indices (count, -1);


        gtk_tree_path_next (path);

        if (!gtk_tree_path_compare (path, p))
          path = gtk_tree_path_new_first ();

        gtk_tree_path_free (p);
      }
      else
        path = gtk_tree_path_new_first ();


      gtk_tree_view_set_cursor (GTK_TREE_VIEW (widget), path, NULL, FALSE);

      /*if (gtk_tree_model_get_iter (model, &iter, path))
         {
         struct tm tm;
         gchar * time;
         time_t viewtime;

         viewtime = cal_get_viewtime ();
         localtime_r (&viewtime, &tm);
         gtk_tree_model_get (model, &iter, COL_TIME, &time, -1);
         strptime (time, DETAIL_TIME_FMT, &tm);
         cal_set_viewtime (mktime(&tm));

         g_free (time);
         } */

    }
    break;
  case GDK_Up:

    if (path)
    {
      GtkTreePath *oldpath = gtk_tree_path_copy (path);
      guint count =
        gtk_tree_model_iter_n_children (GTK_TREE_MODEL (model), NULL);

      gtk_tree_path_prev (path);

      if (!gtk_tree_path_compare (oldpath, path))
        path = gtk_tree_path_new_from_indices (count - 1, -1);

      gtk_tree_view_set_cursor (GTK_TREE_VIEW (widget), path, NULL, FALSE);
      /*if (gtk_tree_model_get_iter (model, &iter, path))
         {
         struct tm tm;
         gchar * time;
         time_t viewtime;

         viewtime = cal_get_viewtime ();
         localtime_r (&viewtime, &tm);
         gtk_tree_model_get (model, &iter, COL_TIME, &time, -1);
         strptime (time, DETAIL_TIME_FMT, &tm);
         cal_set_viewtime (mktime(&tm));

         g_free (time);
         } */

      gtk_tree_path_free (oldpath);
    }

    break;

  }

  day_view_day_indicate_update (view);
  gtk_tree_path_free (path);
  return TRUE;
}


static void
cal_day_view_init (CalDayView *view)
{


/*Set up the widget*/
  view->vbox = gtk_vbox_new (FALSE, 0);

  view->date_indicate = gtk_label_new ("day indicate");
  gtk_box_pack_start (GTK_BOX (view->vbox), view->date_indicate, FALSE, FALSE,
                      0);

  /*Scrolled window */
  view->scrolled_window = gtk_scrolled_window_new (NULL, NULL);
  gtk_box_pack_start (GTK_BOX (view->vbox), view->scrolled_window, TRUE, TRUE,
                      0);
  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (view->scrolled_window),
                                  GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);

/*Treeview*/
  view->treeview_dayevents = create_treeview_dayevents ();
  gtk_container_add (GTK_CONTAINER (view->scrolled_window),
                     view->treeview_dayevents);
  g_signal_connect (G_OBJECT (view->treeview_dayevents), "key_press_event",
                    G_CALLBACK (day_view_key_press_event_cb), view);
  /*g_signal_connect(G_OBJECT (view->treeview_dayevents), "button-press-event",
     G_CALLBACK (day_view_button_press_event_cb), view); */
  /*treeview will receive "row-activated" signal when double clicked */
  /* g_signal_connect (G_OBJECT (view->treeview_dayevents), "row-activated",
     G_CALLBACK (day_view_row_activated_cb), view); */

/*Focus setting*/
  GTK_WIDGET_SET_FLAGS (view->treeview_dayevents, GTK_CAN_FOCUS);
  GTK_WIDGET_SET_FLAGS (view->scrolled_window, GTK_CAN_FOCUS);
  gtk_widget_add_events (GTK_WIDGET (view->treeview_dayevents),
                         GDK_KEY_PRESS_MASK | GDK_KEY_RELEASE_MASK);
  gtk_widget_add_events (GTK_WIDGET (view->scrolled_window),
                         GDK_KEY_PRESS_MASK | GDK_KEY_RELEASE_MASK);

  /*gtk_grab_add (GTK_WIDGET (day_list));*/

/*Scroll adjustment*/
  view->scroll_adjustment =
    gtk_scrolled_window_get_vadjustment (GTK_SCROLLED_WINDOW
                                         (view->scrolled_window));

/*Day events*/
  view->day_events = NULL;


  cal_day_view_update (view);

  gtk_container_add (GTK_CONTAINER (view), GTK_WIDGET (view->vbox));

  return;
}

GtkWidget *
cal_day_view_new (void)
{
  return gtk_widget_new (CAL_TYPE_DAY_VIEW, NULL);
}

void
cal_day_view_destroy (CalDayView *view)
{
  if (view->day_events)
    cal_event_list_destroy (view->day_events);
  view->day_events = NULL;
}

/*
vi:ts=2:nowrap:ai:expandtab
*/

