/*	calendar - calendaring and scheduling application reference implementation
 *
 *  Authors: YANG Lin <lin.yang@orange-ftgroup.com> 
 *  
 *  This software and associated documentation files (the "Software") 
 *  are copyright (C) 2005 LiPS Linux Phone Standards Forum [FranceTelecom] 
 *  All Rights Reserved. 
 *
 *  A copyright license is hereby granted for redistribution and use of 
 *  the Software in source and binary forms, with or without modification, 
 *  provided that the following conditions are met: 
 *  - Redistributions of source code must retain the above copyright notice, 
 *  this copyright license and the following disclaimer. 
 *  - Redistributions in binary form must reproduce the above copyright 
 *   notice, this copyright license and the following disclaimer in the 
 *  documentation and/or other materials provided with the distribution. 
 *  - Neither the name of LiPS  nor the names of its Members may be used 
 *  to endorse or promote products derived from the Software without 
 *  specific prior written permission. 
 *
 *  A patent license for any Necessary Claims owned by Members of LiPS Forum 
 *  to make, have made, use, import, offer to sell, lease and sell or otherwise 
 *  distribute any implementation compliant with the any specification adopted 
 *  by the LiPS Forumcan be obtained from the respective Members on reasonable 
 *  and non-discriminatory terms and conditions and under reciprocity, as 
 *  regulated in more detail in the Internal Policy of the LiPS Forum. 
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDER, ITS MEMBERS AND CONTRIBUTORS 
 *  "AS IS", AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
 *  THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE 
 *  AND NON-INFRINGEMENT ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER, 
 *  ITS MEMBERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
 *  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; 
 *  OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 *  WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 *  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
 *  POSSIBILITY OF SUCH DAMAGE. 
 */

#include <dbus/dbus-glib.h>
#include <dbus/dbus-glib-lowlevel.h>
#include <dbus/dbus.h>
#include "dbus-if.h"
#include "basicframe.h"
#include "shared.h"

static void server_unregistered_func (DBusConnection *connection,
                                      gpointer user_data);
static DBusHandlerResult server_message_func (DBusConnection *connection,
                                              DBusMessage *message,
                                              gpointer user_data);

static DBusObjectPathVTable server_vtable = {
  server_unregistered_func,
  server_message_func,
  NULL,
};

static void
server_unregistered_func (DBusConnection *connection, gpointer user_data)
{
}

static DBusHandlerResult
server_message_func (DBusConnection *connection,
                     DBusMessage *message, gpointer user_data)
{
  GtkWidget *window = (GtkWidget *) user_data;
  if (dbus_message_is_method_call (message,
                                   CALENDAR_DBUS_INTERFACE,
                                   ADDRESSBOOK_DBUS_METHOD_RETRIEVE_CONTACT))
  {
    dbus_invoke = TRUE;
    dbus_int32_t client_serial;
    client_serial = dbus_message_get_serial (message);
    g_log (NULL, G_LOG_LEVEL_MESSAGE, "client_serial is: %d\n", client_serial);
    dbus_message_set_reply_serial (message, client_serial);
    if (reply)                  /*to avoid memory leak*/
      dbus_message_unref (reply);

    reply = dbus_message_new_method_return (message);

    gtk_widget_show_all (window);

    return DBUS_HANDLER_RESULT_HANDLED;
  }
  else
  {
    return DBUS_HANDLER_RESULT_NOT_YET_HANDLED;
  }
}


gboolean
dbus_init (GtkWidget *window)
{
  DBusError error;

  dbus_error_init (&error);
  reply = NULL;

  bus = dbus_bus_get (DBUS_BUS_SESSION, &error);

  if (!bus)
  {
    g_warning ("Failed to connect to the D-BUS daemon: %s", error.message);
    dbus_error_free (&error);
    return 1;
  }


  dbus_bus_request_name (bus, CALENDAR_DBUS_SERVICE, 0, &error);
  if (dbus_error_is_set (&error))
  {
    g_warning ("Failed to acquire addressbook service %s", error.message);
    dbus_error_free (&error);
    return 1;
  }

  if (!dbus_connection_register_object_path (bus,
                                             CALENDAR_DBUS_OBJECT,
                                             &server_vtable, window))
  {
    g_warning ("Failed to register server object with the D-BUS bus daemon");
    return 1;
  }

  dbus_connection_setup_with_g_main (bus, NULL);

  return 0;
}

/*
vi:ts=2:nowrap:ai:expandtab
*/

