/*	calendar - calendaring and scheduling application reference implementation
 *
 *  Authors: YANG Lin <lin.yang@orange-ftgroup.com> 
 *  
 *  This software and associated documentation files (the "Software") 
 *  are copyright (C) 2005 LiPS Linux Phone Standards Forum [FranceTelecom] 
 *  All Rights Reserved. 
 *
 *  A copyright license is hereby granted for redistribution and use of 
 *  the Software in source and binary forms, with or without modification, 
 *  provided that the following conditions are met: 
 *  - Redistributions of source code must retain the above copyright notice, 
 *  this copyright license and the following disclaimer. 
 *  - Redistributions in binary form must reproduce the above copyright 
 *   notice, this copyright license and the following disclaimer in the 
 *  documentation and/or other materials provided with the distribution. 
 *  - Neither the name of LiPS  nor the names of its Members may be used 
 *  to endorse or promote products derived from the Software without 
 *  specific prior written permission. 
 *
 *  A patent license for any Necessary Claims owned by Members of LiPS Forum 
 *  to make, have made, use, import, offer to sell, lease and sell or otherwise 
 *  distribute any implementation compliant with the any specification adopted 
 *  by the LiPS Forumcan be obtained from the respective Members on reasonable 
 *  and non-discriminatory terms and conditions and under reciprocity, as 
 *  regulated in more detail in the Internal Policy of the LiPS Forum. 
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDER, ITS MEMBERS AND CONTRIBUTORS 
 *  "AS IS", AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
 *  THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE 
 *  AND NON-INFRINGEMENT ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER, 
 *  ITS MEMBERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
 *  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; 
 *  OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 *  WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 *  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
 *  POSSIBILITY OF SUCH DAMAGE. 
 */

#include <libintl.h>
#include <time.h>
#include <calenabler/cal.h>
#include <gemwidget/gtktimeedit.h>
#include <gemwidget/gtkdateedit.h>

#include "eventedit.h"
#include "timemanage.h"
#include "calenabler.h"
#include "basicframe.h"



static void cal_event_edit_class_init (CalEventEditClass * klass);
static void cal_event_edit_init (CalEventEdit * view);

GType
cal_event_edit_get_type (void)
{
  static GType eventedit_type = 0;

  if (!eventedit_type)
  {
    static const GTypeInfo eventedit_info = {
      sizeof (CalEventEditClass),
      NULL,                     /* base_init */
      NULL,                     /* base_finalize */
      (GClassInitFunc) cal_event_edit_class_init,
      NULL,                     /* class_finalize */
      NULL,                     /* class_data */
      sizeof (CalEventEdit),
      0,                        /* n_preallocs */
      (GInstanceInitFunc) cal_event_edit_init,
    };

    eventedit_type =
      g_type_register_static (GTK_TYPE_AUTO_SCROLLED_WINDOW, "CalEventEdit",
                              &eventedit_info, 0);
  }

  return eventedit_type;
}

static void
cal_event_edit_class_init (CalEventEditClass * klass)
{
  return;
}

static GtkWidget *
create_combo_adelay (void)
{
  GtkWidget *combo;

  combo = gtk_combo_box_new_text ();
  gtk_combo_box_prepend_text (GTK_COMBO_BOX (combo), "Before 1 week");
  gtk_combo_box_prepend_text (GTK_COMBO_BOX (combo), "Before 1 day");
  gtk_combo_box_prepend_text (GTK_COMBO_BOX (combo), "Before 1 hour");
  gtk_combo_box_prepend_text (GTK_COMBO_BOX (combo), "Before 10 min");
  gtk_combo_box_prepend_text (GTK_COMBO_BOX (combo), "Before 5 min");
  gtk_combo_box_prepend_text (GTK_COMBO_BOX (combo), "Before 1 min");

  gtk_combo_box_set_active (GTK_COMBO_BOX (combo), 0);
  return combo;
}

static GtkWidget *
create_combo_repeat (void)
{
  GtkWidget *combo;
  GtkTreeIter iter;
  GtkCellRenderer *renderer = gtk_cell_renderer_text_new ();
  GtkListStore *store;

  store = gtk_list_store_new (1, G_TYPE_STRING);
  combo = gtk_combo_box_new_with_model (GTK_TREE_MODEL (store));
  gtk_cell_layout_pack_start (GTK_CELL_LAYOUT (combo), renderer, TRUE);
  gtk_cell_layout_set_attributes (GTK_CELL_LAYOUT (combo), renderer,
                                  "text", 0, NULL);

  /*first row */
  gtk_list_store_append (store, &iter);
  gtk_list_store_set (store, &iter, 0, "Every day", -1);

  gtk_list_store_append (store, &iter);
  gtk_list_store_set (store, &iter, 0, "Every week", -1);

  gtk_list_store_append (store, &iter);
  gtk_list_store_set (store, &iter, 0, "Every month", -1);

  gtk_list_store_append (store, &iter);
  gtk_list_store_set (store, &iter, 0, "Every year", -1);

  g_object_unref (store);
  gtk_combo_box_set_active (GTK_COMBO_BOX (combo), 0);

  return combo;
}




static gboolean
date_edit_focus_in_cb (GtkWidget * widget, GdkEventFocus * event,
                       gpointer user_data)
{
  /*printf ("%s\n", __FUNCTION__);
     gtk_widget_grab_focus(GTK_DATE_EDIT (widget)->entry1);

     gtk_widget_child_focus (widget , GTK_DIR_TAB_BACKWARD);  
     gtk_widget_child_focus (widget , GTK_DIR_TAB_BACKWARD);  */

  return FALSE;
}

static gboolean
time_edit_focus_in_cb (GtkWidget * widget, GdkEventFocus * event,
                       gpointer user_data)
{
  /*printf ("%s\n", __FUNCTION__);
     gtk_widget_grab_focus(GTK_TIME_EDIT (widget)->entry1);
     gtk_widget_child_focus (widget , GTK_DIR_TAB_BACKWARD);  */

  return FALSE;
}

static gboolean
entry_focus_in_cb (GtkWidget * widget, GdkEventFocus * event,
                   gpointer user_data)
{
  gtk_entry_set_has_frame (GTK_ENTRY (widget), TRUE);

  return FALSE;
}

static gboolean
entry_focus_out_cb (GtkWidget * widget, GdkEventFocus * event,
                    gpointer user_data)
{
  /*CalEventEdit * view = (CalEventEdit *)user_data;

     printf ("%s\n", __FUNCTION__); */
  gtk_entry_set_has_frame (GTK_ENTRY (widget), FALSE);
  gtk_editable_select_region (GTK_EDITABLE (widget), 0, 0);

  /*if (widget == view->entries[CNT_EDIT_VIEW_PICTURE])
     {
     gchar * picpath = NULL;

     picpath = g_build_filename(abook_env_get_picpath(), gtk_entry_get_text(GTK_ENTRY(widget)), NULL);
     gtk_image_set_from_file(GTK_IMAGE(view->image), picpath);
     g_free(picpath);
     } */

  return FALSE;
}


static gboolean
alarm_toggled_cb (GtkWidget * widget, gpointer user_data)
{
  CalEventEdit *view = (CalEventEdit *) user_data;

  if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (widget)))
  {
    gtk_widget_show (GTK_WIDGET (view->label_adelay));
    gtk_widget_show (GTK_WIDGET (view->combo_adelay));
  }
  else
  {
    gtk_widget_hide (GTK_WIDGET (view->label_adelay));
    gtk_widget_hide (GTK_WIDGET (view->combo_adelay));
  }


  return FALSE;
}


static gboolean
repeat_forever_toggled_cb (GtkWidget * widget, gpointer user_data)
{
  CalEventEdit *view = (CalEventEdit *) user_data;
  time_t viewtime = cal_get_viewtime ();
  struct tm tm;

  localtime_r (&viewtime, &tm);

  if (!gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (widget)))
  {
    gtk_date_edit_set_value (GTK_DATE_EDIT (view->date_edit_rend),
                             tm.tm_year + 1900, tm.tm_mon + 1, tm.tm_mday);
    gtk_widget_show (GTK_WIDGET (view->label_rend));
    gtk_widget_show_all (GTK_WIDGET (view->date_edit_rend));
  }
  else
  {
    gtk_widget_hide (GTK_WIDGET (view->label_rend));
    gtk_widget_hide (GTK_WIDGET (view->date_edit_rend));
  }

  return FALSE;
}

static gboolean
repeat_toggled_cb (GtkWidget * widget, gpointer user_data)
{

  CalEventEdit *view = (CalEventEdit *) user_data;

  if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (widget)))
  {
    gtk_widget_show (GTK_WIDGET (view->label_rtype));
    gtk_widget_show (GTK_WIDGET (view->combo_rtype));
    gtk_widget_show (GTK_WIDGET (view->checkbutton_repeat_forever));
  }
  else
  {
    gtk_widget_hide (GTK_WIDGET (view->label_rtype));
    gtk_widget_hide (GTK_WIDGET (view->combo_rtype));
    gtk_widget_hide (GTK_WIDGET (view->checkbutton_repeat_forever));
  }

  return FALSE;
}

static void
cal_event_edit_init (CalEventEdit * view)
{

  GtkWidget *vbox = NULL;
  GtkWidget *labels[NUM_EVENT_ITEMS];
  GtkWidget *grpbox = NULL;
  char *labelstr[NUM_EVENT_ITEMS] = { "Subject", "Location", "Start date",
    "Start time", "End date", "End time",
    "Alarm delay", "Repeat", "Note"
  };
  gint i;

  vbox = gtk_vbox_new (FALSE, 0);

/*All labels*/
  for (i = EV_SUBJECT; i < NUM_EVENT_ITEMS; i++)
  {
    labels[i] = gtk_label_new (labelstr[i]);
    gtk_misc_set_alignment (GTK_MISC (labels[i]), 0, 0);
  }

/*Subject and Location Entries*/
  for (i = EV_SUBJECT; i <= EV_LOCATION; i++)
  {
    grpbox = gtk_vbox_new (FALSE, 0);

    view->entries[i] = gtk_entry_new ();
    /*gtk_misc_set_alignment(GTK_MISC(labels[i]), 0, 0); */
    gtk_box_pack_start (GTK_BOX (grpbox), labels[i], 0, TRUE, 0);
    gtk_box_pack_start (GTK_BOX (grpbox), view->entries[i], 0, TRUE, 0);

    gtk_entry_set_has_frame (GTK_ENTRY (view->entries[i]), FALSE);
    g_signal_connect (G_OBJECT (view->entries[i]), "focus-in-event",
                      G_CALLBACK (entry_focus_in_cb), NULL);
    g_signal_connect (G_OBJECT (view->entries[i]), "focus-out-event",
                      G_CALLBACK (entry_focus_out_cb), (gpointer) view);

    gtk_widget_set_size_request (GTK_WIDGET (grpbox), -1, 50);
    gtk_box_pack_start (GTK_BOX (vbox), grpbox, 0, TRUE, 0);
  }
/*start and end date/time*/

  grpbox = gtk_vbox_new (FALSE, 0);
  view->date_edit_start = gtk_date_edit_new ();
  gtk_box_pack_start (GTK_BOX (grpbox), labels[EV_START_DATE], 0, TRUE, 0);
  gtk_box_pack_start (GTK_BOX (grpbox), view->date_edit_start, 0, TRUE, 0);
  gtk_date_edit_set_widget_width (GTK_DATE_EDIT (view->date_edit_start), 0, 50,
                                  20, 20, 80);
  gtk_widget_set_size_request (GTK_WIDGET (view->date_edit_start), 200, -1);
  /*gtk_widget_set_size_request (GTK_WIDGET (grpbox), -1, 50);      */
  gtk_box_pack_start (GTK_BOX (vbox), grpbox, 0, TRUE, 0);
  g_signal_connect (G_OBJECT (view->date_edit_start), "focus-in-event",
                    G_CALLBACK (date_edit_focus_in_cb), NULL);


  grpbox = gtk_vbox_new (FALSE, 0);
  view->time_edit_start = gtk_time_edit_new ();
  gtk_box_pack_start (GTK_BOX (grpbox), labels[EV_START_TIME], 0, TRUE, 0);
  gtk_box_pack_start (GTK_BOX (grpbox), view->time_edit_start, 0, TRUE, 0);
  gtk_time_edit_set_widget_width (GTK_TIME_EDIT (view->time_edit_start), 0, 20,
                                  20, 140);
  gtk_widget_set_size_request (GTK_WIDGET (grpbox), -1, 50);
  /*gtk_widget_set_size_request (GTK_WIDGET (view->time_edit_start), 200, -1); */
  gtk_box_pack_start (GTK_BOX (vbox), grpbox, 0, TRUE, 0);
  g_signal_connect (G_OBJECT (view->time_edit_start), "focus-in-event",
                    G_CALLBACK (time_edit_focus_in_cb), NULL);

  grpbox = gtk_vbox_new (FALSE, 0);
  view->date_edit_end = gtk_date_edit_new ();
  gtk_box_pack_start (GTK_BOX (grpbox), labels[EV_END_DATE], 0, TRUE, 0);
  gtk_box_pack_start (GTK_BOX (grpbox), view->date_edit_end, 0, TRUE, 0);
  gtk_date_edit_set_widget_width (GTK_DATE_EDIT (view->date_edit_end), 0, 50,
                                  20, 20, 80);
  gtk_widget_set_size_request (GTK_WIDGET (grpbox), -1, 50);
  gtk_box_pack_start (GTK_BOX (vbox), grpbox, 0, TRUE, 0);
  g_signal_connect (G_OBJECT (view->date_edit_end), "focus-in-event",
                    G_CALLBACK (date_edit_focus_in_cb), NULL);


  grpbox = gtk_vbox_new (FALSE, 0);
  view->time_edit_end = gtk_time_edit_new ();
  gtk_box_pack_start (GTK_BOX (grpbox), labels[EV_END_TIME], 0, TRUE, 0);
  gtk_box_pack_start (GTK_BOX (grpbox), view->time_edit_end, 0, TRUE, 0);
  gtk_time_edit_set_widget_width (GTK_TIME_EDIT (view->time_edit_end), 0, 20,
                                  20, 140);
  gtk_widget_set_size_request (GTK_WIDGET (grpbox), -1, 50);
  gtk_box_pack_start (GTK_BOX (vbox), grpbox, 0, TRUE, 0);
  g_signal_connect (G_OBJECT (view->time_edit_end), "focus-in-event",
                    G_CALLBACK (time_edit_focus_in_cb), NULL);
/*Alarm check button */
  grpbox = gtk_vbox_new (FALSE, 0);

  view->checkbutton_alarm = gtk_check_button_new_with_label ("Alarm");
  gtk_box_pack_start (GTK_BOX (grpbox), view->checkbutton_alarm, 0, TRUE, 0);

  view->label_adelay = gtk_label_new ("Alarm delay:");
  gtk_misc_set_alignment (GTK_MISC (view->label_adelay), 0, 0);

  view->combo_adelay = create_combo_adelay ();

  gtk_box_pack_start (GTK_BOX (grpbox), view->label_adelay, 0, TRUE, 0);
  gtk_box_pack_start (GTK_BOX (grpbox), view->combo_adelay, 0, TRUE, 0);

  g_signal_connect (G_OBJECT (view->checkbutton_alarm), "toggled",
                    G_CALLBACK (alarm_toggled_cb), view);

  gtk_box_pack_start (GTK_BOX (vbox), grpbox, 0, TRUE, 0);

  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (view->checkbutton_alarm),
                                FALSE);

/*Repeat check button */
  grpbox = gtk_vbox_new (FALSE, 0);

  view->checkbutton_repeat = gtk_check_button_new_with_label ("Repeat ");
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (view->checkbutton_repeat),
                                FALSE);
  gtk_box_pack_start (GTK_BOX (grpbox), view->checkbutton_repeat, 0, TRUE, 0);

  /*Repeat combobox */
  view->label_rtype = gtk_label_new ("Repeat type:");
  gtk_misc_set_alignment (GTK_MISC (view->label_rtype), 0, 0);

  view->combo_rtype = create_combo_repeat ();
  view->checkbutton_repeat_forever =
    gtk_check_button_new_with_label ("Repeat forever");

  gtk_box_pack_start (GTK_BOX (grpbox), view->label_rtype, 0, TRUE, 0);
  gtk_box_pack_start (GTK_BOX (grpbox), view->combo_rtype, 0, TRUE, 0);
  gtk_box_pack_start (GTK_BOX (grpbox), view->checkbutton_repeat_forever, 0,
                      TRUE, 0);

  /*Repeat forever combobox */
  view->label_rend = gtk_label_new ("Repeat end");
  gtk_misc_set_alignment (GTK_MISC (view->label_rend), 0, 0);

  view->date_edit_rend = gtk_date_edit_new ();

  gtk_date_edit_set_widget_width (GTK_DATE_EDIT (view->date_edit_rend), 0, 50,
                                  20, 20, 80);
  gtk_box_pack_start (GTK_BOX (grpbox), view->label_rend, 0, TRUE, 0);
  gtk_box_pack_start (GTK_BOX (grpbox), view->date_edit_rend, 0, TRUE, 0);

  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON
                                (view->checkbutton_repeat_forever), TRUE);

  g_signal_connect (G_OBJECT (view->checkbutton_repeat_forever), "toggled",
                    G_CALLBACK (repeat_forever_toggled_cb), view);

  g_signal_connect (G_OBJECT (view->checkbutton_repeat), "toggled",
                    G_CALLBACK (repeat_toggled_cb), view);

  gtk_box_pack_start (GTK_BOX (vbox), grpbox, 0, TRUE, 0);


/*Note*/
  grpbox = gtk_vbox_new (FALSE, 0);

  gtk_box_pack_start (GTK_BOX (grpbox), labels[EV_NOTE], 0, TRUE, 0);
  view->textview_note = gtk_entry_new ();
  gtk_box_pack_start (GTK_BOX (grpbox), view->textview_note, 0, TRUE, 0);

  gtk_entry_set_has_frame (GTK_ENTRY (view->textview_note), FALSE);
  g_signal_connect (G_OBJECT (view->textview_note), "focus-in-event",
                    G_CALLBACK (entry_focus_in_cb), NULL);
  g_signal_connect (G_OBJECT (view->textview_note), "focus-out-event",
                    G_CALLBACK (entry_focus_out_cb), (gpointer) view);
/*
  view->textview_note = create_textview_note ();
  gtk_box_pack_start(GTK_BOX(grpbox), view->textview_note , 0, TRUE, 0);  
  
  gtk_text_view_set_wrap_mode (GTK_TEXT_VIEW (view->textview_note), 
                                GTK_WRAP_WORD);
  gtk_text_view_set_editable (GTK_TEXT_VIEW (view->textview_note), TRUE);
  g_signal_connect (G_OBJECT(view->textview_note), "focus-in-event", 
                    G_CALLBACK(textview_focus_in_cb), NULL);

  gtk_widget_set_size_request (GTK_WIDGET (view->textview_note), -1, 88);
*/
  gtk_box_pack_start (GTK_BOX (vbox), grpbox, 0, TRUE, 0);


  gtk_auto_scrolled_window_add_with_viewpoert (GTK_AUTO_SCROLLED_WINDOW (view),
                                               vbox);

  view->event = NULL;

  return;
}

GtkWidget *
cal_event_edit_new (void)
{
  return gtk_widget_new (CAL_TYPE_EVENT_EDIT, NULL);
}



void
cal_edit_event (CalEventEdit * view, cal_event_t * ev)
{

  time_t viewtime = cal_get_viewtime ();
  struct tm tm;
  time_t time;

  g_assert (CAL_IS_EVENT_EDIT (view));

  gtk_widget_grab_focus (GTK_WIDGET (view->entries[EV_SUBJECT]));

  if (!ev)                      //new_event
  {
    view->event = NULL;

    localtime_r (&viewtime, &tm);

    gtk_editable_delete_text (GTK_EDITABLE (view->entries[EV_SUBJECT]), 0, -1);
    gtk_editable_delete_text (GTK_EDITABLE (view->entries[EV_LOCATION]), 0, -1);

    gtk_date_edit_set_value (GTK_DATE_EDIT (view->date_edit_start),
                             tm.tm_year + 1900, tm.tm_mon + 1, tm.tm_mday);

    gtk_time_edit_set_value (GTK_TIME_EDIT (view->time_edit_start),
                             tm.tm_hour, tm.tm_min);

    gtk_date_edit_set_value (GTK_DATE_EDIT (view->date_edit_end),
                             tm.tm_year + 1900, tm.tm_mon + 1, tm.tm_mday);

    gtk_time_edit_set_value (GTK_TIME_EDIT (view->time_edit_end),
                             tm.tm_hour, tm.tm_min);

    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (view->checkbutton_alarm),
                                  FALSE);
    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (view->checkbutton_repeat),
                                  FALSE);
    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON
                                  (view->checkbutton_repeat_forever), TRUE);

    gtk_widget_hide (GTK_WIDGET (view->label_adelay));
    gtk_widget_hide (GTK_WIDGET (view->combo_adelay));

    gtk_widget_hide (GTK_WIDGET (view->label_rtype));
    gtk_widget_hide (GTK_WIDGET (view->combo_rtype));
    gtk_widget_hide (GTK_WIDGET (view->checkbutton_repeat_forever));

    gtk_widget_hide (GTK_WIDGET (view->label_rend));
    gtk_widget_hide (GTK_WIDGET (view->date_edit_rend));

  }
  else                          //edit_event
  {

    guint index = 0, delay = 0;

    view->event = ev;


    if (event_get_description (ev))
      gtk_entry_set_text (GTK_ENTRY (view->entries[EV_SUBJECT]),
                          event_get_description (ev));
    if (event_get_location (ev))
      gtk_entry_set_text (GTK_ENTRY (view->entries[EV_LOCATION]),
                          event_get_location (ev));

    time = event_get_start (ev);
    localtime_r (&time, &tm);

    gtk_date_edit_set_value (GTK_DATE_EDIT (view->date_edit_start),
                             tm.tm_year + 1900, tm.tm_mon + 1, tm.tm_mday);

    gtk_time_edit_set_value (GTK_TIME_EDIT (view->time_edit_start),
                             tm.tm_hour, tm.tm_min);

    time = event_get_end (ev);
    localtime_r (&time, &tm);

    gtk_date_edit_set_value (GTK_DATE_EDIT (view->date_edit_end),
                             tm.tm_year + 1900, tm.tm_mon + 1, tm.tm_mday);

    gtk_time_edit_set_value (GTK_TIME_EDIT (view->time_edit_end),
                             tm.tm_hour, tm.tm_min);

    if (event_get_has_alarm (ev))
    {
      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (view->checkbutton_alarm),
                                    TRUE);
      delay = event_get_alarm_delay (ev);
      switch (delay)
      {
      case 60:
        index = 0;
        break;
      case 300:
        index = 1;
        break;
      case 600:
        index = 2;
        break;
      case 3600:
        index = 3;
        break;
      case 86400:
        index = 4;
        break;
      case 604800:
        index = 5;
        break;
      }
      gtk_combo_box_set_active (GTK_COMBO_BOX (view->combo_adelay), index);
    }
    else
    {
      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (view->checkbutton_alarm),
                                    FALSE);
      gtk_widget_hide (GTK_WIDGET (view->label_adelay));
      gtk_widget_hide (GTK_WIDGET (view->combo_adelay));
    }

    if (event_get_repeat_type (ev) == REPEAT_NONE)
    {
      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON
                                    (view->checkbutton_repeat), FALSE);
      gtk_widget_hide (GTK_WIDGET (view->label_rtype));
      gtk_widget_hide (GTK_WIDGET (view->combo_rtype));
      gtk_widget_hide (GTK_WIDGET (view->checkbutton_repeat_forever));

      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON
                                    (view->checkbutton_repeat_forever), TRUE);
      gtk_widget_hide (GTK_WIDGET (view->label_rend));
      gtk_widget_hide (GTK_WIDGET (view->date_edit_rend));
    }
    else
    {
      guint index;

      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON
                                    (view->checkbutton_repeat), TRUE);
      index = event_get_repeat_type (ev);
#ifdef DEBUG
      printf ("%s:combobox set active: %d\n", __FUNCTION__, index);
#endif
      gtk_combo_box_set_active (GTK_COMBO_BOX (view->combo_rtype), index - 1);

      if (!event_get_repeat_forever (ev))
      {

        gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON
                                      (view->checkbutton_repeat_forever),
                                      FALSE);
        time = event_get_repeat_end (ev);
        localtime_r (&time, &tm);
        gtk_date_edit_set_value (GTK_DATE_EDIT (view->date_edit_rend),
                                 tm.tm_year + 1900, tm.tm_mon + 1, tm.tm_mday);
      }
      else
      {
        gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON
                                      (view->checkbutton_repeat_forever), TRUE);
        gtk_widget_hide (GTK_WIDGET (view->label_rend));
        gtk_widget_hide (GTK_WIDGET (view->date_edit_rend));
      }
    }

    if (event_get_notes (ev))
      gtk_entry_set_text (GTK_ENTRY (view->textview_note),
                          event_get_notes (ev));
  }

}


gboolean
cal_save_event (CalEventEdit * view)
{
  cal_eventbook_t *book = cal_enabler_get_book ();
  gchar *buf = NULL;
  time_t now = time (NULL);
  time_t starttime, endtime, rend;
  struct tm tm;
  gchar *uid = NULL;

  g_assert (CAL_IS_EVENT_EDIT (view));


  localtime_r (&now, &tm);

  if (!view->event)
    view->event = event_new ();
#ifdef DEBUG
  else
    cal_print_event (view->event);
#endif



  buf = g_strdup (gtk_entry_get_text (GTK_ENTRY (view->entries[EV_SUBJECT])));

  if (g_utf8_strlen (buf, -1) == 0)
  {
    GtkWidget *dialog;

    dialog = gtk_message_dialog_new (GTK_WINDOW (self.window),
                                     GTK_DIALOG_MODAL |
                                     GTK_DIALOG_DESTROY_WITH_PARENT,
                                     GTK_MESSAGE_WARNING, GTK_BUTTONS_CLOSE,
                                     "cal_event_t subject shouldn't be NULL\n");
    gtk_window_set_decorated (GTK_WINDOW (dialog), TRUE);
    gtk_window_set_default_size (GTK_WINDOW (dialog), 200, 300);
    /*g_signal_connect_swapped (dialog,
       "response", 
       G_CALLBACK (gtk_widget_destroy),
       dialog); */
    /*gtk_widget_show_all (dialog);

       gtk_widget_destroy (dialog); */
    gtk_dialog_run (GTK_DIALOG (dialog));
    gtk_widget_destroy (dialog);

    return FALSE;
  }

  event_set_description (view->event, buf);
  g_free (buf);

  buf = g_strdup (gtk_entry_get_text (GTK_ENTRY (view->entries[EV_LOCATION])));
  event_set_location (view->event, buf);
  g_free (buf);

  tm.tm_year =
    gtk_date_edit_get_year (GTK_DATE_EDIT (view->date_edit_start)) - 1900;
  tm.tm_mon = gtk_date_edit_get_mon (GTK_DATE_EDIT (view->date_edit_start)) - 1;
  tm.tm_mday = gtk_date_edit_get_day (GTK_DATE_EDIT (view->date_edit_start));
  tm.tm_hour = gtk_time_edit_get_hour (GTK_TIME_EDIT (view->time_edit_start));
  tm.tm_min = gtk_time_edit_get_min (GTK_TIME_EDIT (view->time_edit_start));
  tm.tm_sec = 0;

  starttime = mktime (&tm);

  if (starttime < now)
  {
    GtkWidget *dialog;

    dialog = gtk_message_dialog_new (GTK_WINDOW (self.window),
                                     GTK_DIALOG_MODAL |
                                     GTK_DIALOG_DESTROY_WITH_PARENT,
                                     GTK_MESSAGE_WARNING, GTK_BUTTONS_CLOSE,
                                     "cal_event_t start in the past\n");
    gtk_window_set_decorated (GTK_WINDOW (dialog), TRUE);
    gtk_dialog_run (GTK_DIALOG (dialog));
    gtk_widget_destroy (dialog);

    return FALSE;
  }
  event_set_start (view->event, starttime);

  tm.tm_year =
    gtk_date_edit_get_year (GTK_DATE_EDIT (view->date_edit_end)) - 1900;
  tm.tm_mon = gtk_date_edit_get_mon (GTK_DATE_EDIT (view->date_edit_end)) - 1;
  tm.tm_mday = gtk_date_edit_get_day (GTK_DATE_EDIT (view->date_edit_end));
  tm.tm_hour = gtk_time_edit_get_hour (GTK_TIME_EDIT (view->time_edit_end));
  tm.tm_min = gtk_time_edit_get_min (GTK_TIME_EDIT (view->time_edit_end));
  tm.tm_sec = 0;

  endtime = mktime (&tm);
  if (endtime < starttime)
  {
    GtkWidget *dialog;

    dialog = gtk_message_dialog_new (GTK_WINDOW (self.window),
                                     GTK_DIALOG_MODAL |
                                     GTK_DIALOG_DESTROY_WITH_PARENT,
                                     GTK_MESSAGE_WARNING, GTK_BUTTONS_CLOSE,
                                     "cal_event_t start later than end\n");
    gtk_window_set_decorated (GTK_WINDOW (dialog), TRUE);
    gtk_dialog_run (GTK_DIALOG (dialog));
    gtk_widget_destroy (dialog);

    return FALSE;
  }
  event_set_end (view->event, endtime);

  if (gtk_toggle_button_get_active
      (GTK_TOGGLE_BUTTON (view->checkbutton_alarm)))
  {
    guint index = 0, delay = 0;

    event_set_has_alarm (view->event, TRUE);
    index = gtk_combo_box_get_active (GTK_COMBO_BOX (view->combo_adelay));

    switch (index)
    {
    case 0:
      delay = 60;
      break;
    case 1:
      delay = 300;              //5 * 60
      break;
    case 2:
      delay = 600;              /* 10*60 */
      break;
    case 3:
      delay = 3600;
      break;
      /*case 4:  
         delay = 86400; //24 * 3600
         break;
         case 5:
         delay = 604800; //7 * 24 * 3600
         break; */
    }
    event_set_alarm_delay (view->event, delay);
  }
  else
  {
    event_set_has_alarm (view->event, FALSE);
  }

  if (gtk_toggle_button_get_active
      (GTK_TOGGLE_BUTTON (view->checkbutton_repeat)))
  {
    guint index = 0;

    index = gtk_combo_box_get_active (GTK_COMBO_BOX (view->combo_rtype));
#ifdef DEBUG
    printf ("%s: index=%d\n", __FUNCTION__, index);
#endif
    event_set_repeat_type (view->event, index + 1);

    /*For repeat events, the start and end time must be on the same day */
    if (index != 0)
    {
      time_t start, end;
      struct tm tm_start, tm_end;

      end = event_get_end (view->event);
      start = event_get_start (view->event);

      localtime_r (&start, &tm_start);
      localtime_r (&end, &tm_end);

      if (!((tm_start.tm_year == tm_end.tm_year) 
          &&(tm_start.tm_mon == tm_end.tm_mon) 
          &&(tm_start.tm_mday == tm_end.tm_mday)))
      {
        GtkWidget *dialog;

        dialog = gtk_message_dialog_new (GTK_WINDOW (self.window),
                                         GTK_DIALOG_MODAL |
                                         GTK_DIALOG_DESTROY_WITH_PARENT,
                                         GTK_MESSAGE_WARNING, GTK_BUTTONS_CLOSE,
                                         "The repeating event's start date and the end date must be the same\n");
        gtk_window_set_decorated (GTK_WINDOW (dialog), TRUE);
        gtk_dialog_run (GTK_DIALOG (dialog));
        gtk_widget_destroy (dialog);

        return FALSE;
      }
    }

    switch (index)
    {
    case 1:                    /*REPEAT_WEEKLY */
      event_set_repeat_weekdays (view->event, 0x1 << (tm.tm_wday - 1));
      break;
    case 2:                    /*REPEAT_MONTHLY*/
      event_set_repeat_monthdays (view->event, 0x1 << (tm.tm_mday - 1));
      break;
    }
    event_set_repeat_frequency (view->event, 1);

    if (!gtk_toggle_button_get_active
        (GTK_TOGGLE_BUTTON (view->checkbutton_repeat_forever)))
    {
      event_set_repeat_forever (view->event, FALSE);

      tm.tm_year =
        gtk_date_edit_get_year (GTK_DATE_EDIT (view->date_edit_rend)) - 1900;
      tm.tm_mon =
        gtk_date_edit_get_mon (GTK_DATE_EDIT (view->date_edit_rend)) - 1;
      tm.tm_mday = gtk_date_edit_get_day (GTK_DATE_EDIT (view->date_edit_rend));

      rend = mktime (&tm);
      if (rend < endtime)
      {
        GtkWidget *dialog;

        dialog = gtk_message_dialog_new (GTK_WINDOW (self.window),
                                         GTK_DIALOG_MODAL |
                                         GTK_DIALOG_DESTROY_WITH_PARENT,
                                         GTK_MESSAGE_WARNING, GTK_BUTTONS_CLOSE,
                                         "cal_event_t repeat end before event end\n");
        gtk_window_set_decorated (GTK_WINDOW (dialog), FALSE);
        gtk_dialog_run (GTK_DIALOG (dialog));
        gtk_widget_destroy (dialog);

        return FALSE;
      }
      event_set_repeat_end (view->event, rend);
    }
    else
    {
      event_set_repeat_forever (view->event, TRUE);
    }

    event_set_has_exception (view->event, FALSE);
  }
  else
    event_set_repeat_type (view->event, REPEAT_NONE);

  buf = g_strdup (gtk_entry_get_text (GTK_ENTRY (view->textview_note)));
  event_set_notes (view->event, buf);
  g_free (buf);

  uid = event_get_uid (view->event);

  if (!uid)
  {
    eventbook_add_event (book, view->event);
    event_free (view->event);
    view->event = NULL;

  }
  else
  {
#ifdef DEBUG
    /*cal_print_event (view->event);*/
#endif
    eventbook_update_event (book, view->event);
    view->event = NULL;

  }

  return TRUE;
}


/*
vi:ts=2:nowrap:ai:expandtab
*/

