/*	calendar - calendaring and scheduling application reference implementation
 *
 *  Authors: YANG Lin <lin.yang@orange-ftgroup.com> 
 *  
 *  This software and associated documentation files (the "Software") 
 *  are copyright (C) 2005 LiPS Linux Phone Standards Forum [FranceTelecom] 
 *  All Rights Reserved. 
 *
 *  A copyright license is hereby granted for redistribution and use of 
 *  the Software in source and binary forms, with or without modification, 
 *  provided that the following conditions are met: 
 *  - Redistributions of source code must retain the above copyright notice, 
 *  this copyright license and the following disclaimer. 
 *  - Redistributions in binary form must reproduce the above copyright 
 *   notice, this copyright license and the following disclaimer in the 
 *  documentation and/or other materials provided with the distribution. 
 *  - Neither the name of LiPS  nor the names of its Members may be used 
 *  to endorse or promote products derived from the Software without 
 *  specific prior written permission. 
 *
 *  A patent license for any Necessary Claims owned by Members of LiPS Forum 
 *  to make, have made, use, import, offer to sell, lease and sell or otherwise 
 *  distribute any implementation compliant with the any specification adopted 
 *  by the LiPS Forumcan be obtained from the respective Members on reasonable 
 *  and non-discriminatory terms and conditions and under reciprocity, as 
 *  regulated in more detail in the Internal Policy of the LiPS Forum. 
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDER, ITS MEMBERS AND CONTRIBUTORS 
 *  "AS IS", AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
 *  THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE 
 *  AND NON-INFRINGEMENT ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER, 
 *  ITS MEMBERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
 *  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; 
 *  OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 *  WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 *  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
 *  POSSIBILITY OF SUCH DAMAGE. 
 */


#include <gtk/gtk.h>
#include <time.h>
#include <gpe/gpeskwindow.h>
#include <gdk/gdk.h>
#include <gemwidget/gem.h>

#include "timemanage.h"
#include "basicframe.h"
#include "gotodate.h"
#include "calenabler.h"



void
gotodate_softkey_activated_cb (GtkWidget *widget, gint key_number)
{

  GtkWidget *entry = g_object_get_data (G_OBJECT (widget), "entry_goto");
  GtkNotebook *notebook;
  struct tm tm;
  time_t viewtime = cal_get_viewtime ();



  notebook = (GtkNotebook *) self.notebook;

  localtime_r (&viewtime, &tm);

  if (key_number == SK_LEFT)
  {
    tm.tm_year = gtk_date_edit_get_year (GTK_DATE_EDIT (entry)) - 1900;
    tm.tm_mon = gtk_date_edit_get_mon (GTK_DATE_EDIT (entry)) - 1;
    tm.tm_mday = gtk_date_edit_get_day (GTK_DATE_EDIT (entry));

    viewtime = mktime (&tm);


    if ((viewtime >= 0) && (viewtime < 2145888000))
    {
      cal_set_viewtime (viewtime);
      update_all_views ();

    }
/*The date_edit widget has checked the user input.*/
#if 0
    if ((viewtime < 0) || (viewtime >= 2145888000))
    {
      GtkWidget *dialog;

      dialog = gtk_message_dialog_new (GTK_WINDOW (self.window),
                                       GTK_DIALOG_MODAL |
                                       GTK_DIALOG_DESTROY_WITH_PARENT,
                                       GTK_MESSAGE_WARNING, GTK_BUTTONS_CLOSE,
                                       "Only display 1976 ~ 2037\n");
      gtk_window_set_decorated (GTK_WINDOW (dialog), TRUE);
      gtk_dialog_run (GTK_DIALOG (dialog));
      gtk_widget_destroy (dialog);

    }
#endif
  }
  gtk_widget_destroy (widget);
}


gboolean
cal_delete_before_date (time_t time)
{
  cal_eventbook_t *book = cal_enabler_get_book ();
  cal_event_iter_t *eviter = event_iterator_new ();
  guint i;
  gboolean flag = FALSE;

  /*printf ("%s: delete before time:%d\n", __FUNCTION__, time);*/

  eventbook_get_events_for_period (book, eviter, 0, time);

  for (i = 0; i < event_iterator_size (eviter); i++)
  {
    cal_event_t *ev;
    gchar *uid;

    flag = TRUE;
    ev = event_iterator_current (eviter);
    /*cal_print_event (ev);*/
    uid = event_get_uid (ev);
    eventbook_remove_event (book, uid);

    event_iterator_next (eviter);

    g_free (uid);
  }
  return flag;
}

void
beforedate_softkey_activated_cb (GtkWidget *widget, gint key_number)
{

  GtkWidget *entry = g_object_get_data (G_OBJECT (widget), "entry_goto");
  gchar *timebuf = NULL;
  struct tm tm;
  time_t beforetime = cal_get_viewtime ();

  localtime_r (&beforetime, &tm);

  if (key_number == SK_LEFT)
  {
    tm.tm_year = gtk_date_edit_get_year (GTK_DATE_EDIT (entry)) - 1900;
    tm.tm_mon = gtk_date_edit_get_mon (GTK_DATE_EDIT (entry)) - 1;
    tm.tm_mday = gtk_date_edit_get_day (GTK_DATE_EDIT (entry));

    beforetime = mktime (&tm);
    /*printf ("%s: delete before time:%d\n", __FUNCTION__, beforetime);*/
    if (cal_delete_before_date (beforetime))
      update_all_views ();
  }

  gtk_widget_destroy (widget);

  g_free (timebuf);

}

GtkWidget *
cal_go_to_date_new (guint flag)
{
  GtkWidget *dialog;
  GtkWidget *frame;
  GtkWidget *vbox;
  GtkWidget *label;
  GtkWidget *eventbox;
  GtkWidget *entry;
  time_t nowtime;
  struct tm tm;


  time (&nowtime);
  localtime_r (&nowtime, &tm);
  dialog = gpe_skwindow_new ();
  gem_sk_window_set_dialog (GPE_SKWINDOW (dialog));
  /*Set the dialog transient for the parent window */
  gtk_window_set_transient_for (dialog, self.window);

  frame = gtk_frame_new (NULL);
  gtk_container_add (GTK_CONTAINER (dialog), frame);

  vbox = gtk_vbox_new (FALSE, 0);
  gtk_container_add (GTK_CONTAINER (frame), vbox);
  eventbox = gtk_event_box_new ();

  if (flag == GO_TO_DATE)
    label = gtk_label_new ("Go to date:");

  else if (flag == BEFORE_DATE)
    label = gtk_label_new ("Before date:");

  gtk_misc_set_alignment (GTK_MISC (label), 0, 0);
  gtk_box_pack_start (GTK_BOX (vbox), eventbox, FALSE, FALSE, 0);
  gtk_container_add (GTK_CONTAINER (eventbox), label);


  entry = gtk_date_edit_new ();
  gtk_date_edit_set_value (GTK_DATE_EDIT (entry), tm.tm_year + 1900,
                           tm.tm_mon + 1, tm.tm_mday);
  gtk_box_pack_start (GTK_BOX (vbox), entry, FALSE, FALSE, 0);
  gtk_date_edit_set_widget_width (GTK_DATE_EDIT (entry), 70, 35, 20, 20, 70);



  if (flag == GO_TO_DATE)
    g_signal_connect_swapped (dialog, "softkey_activated",
                              G_CALLBACK (gotodate_softkey_activated_cb),
                              dialog);
  else if (flag == BEFORE_DATE)
  {

    g_signal_connect_swapped (dialog, "softkey_activated",
                              G_CALLBACK (beforedate_softkey_activated_cb),
                              dialog);
  }

  g_object_set_data (G_OBJECT (dialog), "entry_goto", (gpointer) entry);

  gem_sk_dialog_set_position (GTK_WINDOW (self.window), GPE_SKWINDOW (dialog),
                              20);

  if (1)
  {

    GdkColor blue;
#if 0
    GdkPixbuf *gdkpixbuf;
    GdkPixmap *bgPixmap;
    GdkBitmap *bgMask;
    GError *error = NULL;
    GtkStyle *gtkStyle;

    gdkpixbuf =
      gdk_pixbuf_new_from_file
      ("/home/yanglin/Projects/calcalendar/src/top_bar.png", &error);
    if (error != NULL)
    {
      g_print ("unabler to find bg file\n");
      g_error_free (error);
    }
    gdk_pixbuf_render_pixmap_and_mask (gdkpixbuf, &bgPixmap, &bgMask, 1);
    if (bgPixmap == NULL)
      g_print ("no pixmap created\n");

    gtkStyle = gtk_rc_get_style (eventbox);
    /*gtkStyle->bg_pixmap[0] = bgPixmap;*/


#endif

    blue.red = 14906;
    blue.green = 35980;
    blue.blue = 55769;



    gtk_widget_modify_bg (eventbox, GTK_STATE_NORMAL, &blue);
#if 0
    gtk_widget_modify_style (eventbox, gtkStyle);
    draw_bg_pixmap (dialog);
#endif
  }

  gtk_widget_show_all (dialog);

  return dialog;
}

void
draw_bg_pixmap (GtkWidget *widget)
{
  GdkPixbuf *gdkpixbuf;
  GdkPixmap *bgPixmap;
  GdkBitmap *bgMask;
  GError *error = NULL;
  GtkStyle *gtkStyle;

  gdkpixbuf =
    gdk_pixbuf_new_from_file
    ("/home/yanglin/Projects/calcalendar/src/top_bar.png", &error);
  if (error != NULL)
  {
    g_print ("unabler to find bg file\n");
    g_error_free (error);
  }
  gdk_pixbuf_render_pixmap_and_mask (gdkpixbuf, &bgPixmap, &bgMask, 1);
  if (bgPixmap == NULL)
    g_print ("no pixmap created\n");

  gtkStyle = gtk_rc_get_style (widget);
  gtkStyle->bg_pixmap[0] = bgPixmap;
  gtk_style_set_background (gtkStyle, widget->window, GTK_STATE_NORMAL);
}

/*
vi:ts=2:nowrap:ai:expandtab
*/

