/*	calendar - calendaring and scheduling application reference implementation
 *
 *  Authors: YANG Lin <lin.yang@orange-ftgroup.com> 
 *  
 *  This software and associated documentation files (the "Software") 
 *  are copyright (C) 2005 LiPS Linux Phone Standards Forum [FranceTelecom] 
 *  All Rights Reserved. 
 *
 *  A copyright license is hereby granted for redistribution and use of 
 *  the Software in source and binary forms, with or without modification, 
 *  provided that the following conditions are met: 
 *  - Redistributions of source code must retain the above copyright notice, 
 *  this copyright license and the following disclaimer. 
 *  - Redistributions in binary form must reproduce the above copyright 
 *   notice, this copyright license and the following disclaimer in the 
 *  documentation and/or other materials provided with the distribution. 
 *  - Neither the name of LiPS  nor the names of its Members may be used 
 *  to endorse or promote products derived from the Software without 
 *  specific prior written permission. 
 *
 *  A patent license for any Necessary Claims owned by Members of LiPS Forum 
 *  to make, have made, use, import, offer to sell, lease and sell or otherwise 
 *  distribute any implementation compliant with the any specification adopted 
 *  by the LiPS Forumcan be obtained from the respective Members on reasonable 
 *  and non-discriminatory terms and conditions and under reciprocity, as 
 *  regulated in more detail in the Internal Policy of the LiPS Forum. 
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDER, ITS MEMBERS AND CONTRIBUTORS 
 *  "AS IS", AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
 *  THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE 
 *  AND NON-INFRINGEMENT ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER, 
 *  ITS MEMBERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
 *  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; 
 *  OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 *  WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 *  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
 *  POSSIBILITY OF SUCH DAMAGE. 
 */

/*the usual headers*/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <langinfo.h>

/*gtk includes*/
#include <gtk/gtk.h>
#include <glib.h>

/*internationalisation aka i18n*/
#include <libintl.h>
#include <locale.h>

/*gpe and cal enabler includes*/
#include <gpe/gpeskwindow.h>
#include <calenabler/cal.h>

#include "shared.h"
#include "basicframe.h"
#include "dbus-if.h"
#include "monthview.h"
#include "weekview.h"
#include "dayview.h"
#include "calenabler.h"
#include "timemanage.h"
#include "eventedit.h"
#include "gotodate.h"

#define _(x) gettext(x)

SELF self;



void
update_all_views ()
{

  GtkWidget *view;

  view =
    gtk_notebook_get_nth_page (GTK_NOTEBOOK (self.notebook), DAY_VIEW_PAGE);
  if (CAL_IS_DAY_VIEW (view))
    cal_day_view_update (CAL_DAY_VIEW (view));

  view =
    gtk_notebook_get_nth_page (GTK_NOTEBOOK (self.notebook), MONTH_VIEW_PAGE);
  if (CAL_IS_MONTH_VIEW (view))
    cal_month_view_update (CAL_MONTH_VIEW (view), OTHER);
}

void
switch_to_day_view ()
{

  GtkWidget *view;

  gtk_notebook_set_current_page (GTK_NOTEBOOK (self.notebook), DAY_VIEW_PAGE);
  view =
    gtk_notebook_get_nth_page (GTK_NOTEBOOK (self.notebook), DAY_VIEW_PAGE);
  if (CAL_IS_DAY_VIEW (view))
    cal_day_view_update (CAL_DAY_VIEW (view));
}


void
switch_to_new_event ()
{

  GtkWidget *view;

  /*if (!self.newevent)
     {
     self.newevent    = cal_event_edit_new ();
     gtk_notebook_insert_page (GTK_NOTEBOOK (self.notebook), self.newevent  ,   NULL, EVENT_EDIT_PAGE);
     gtk_widget_show_all (self.newevent);
     } */

  gtk_notebook_set_current_page (GTK_NOTEBOOK (self.notebook), EVENT_EDIT_PAGE);
  view =
    gtk_notebook_get_nth_page (GTK_NOTEBOOK (self.notebook), EVENT_EDIT_PAGE);
  if (CAL_IS_EVENT_EDIT (view))
    cal_edit_event (CAL_EVENT_EDIT (view), NULL);
  gtk_window_set_focus (GTK_WINDOW (self.window),
                        CAL_EVENT_EDIT (view)->entries[EV_SUBJECT]);
}

void
switch_to_edit_event (cal_event_t *ev)
{
  GtkWidget *view;

  /*if (!self.newevent)
     {
     self.newevent    = cal_event_edit_new ();
     gtk_notebook_insert_page (GTK_NOTEBOOK (self.notebook), self.newevent  ,   NULL, EVENT_EDIT_PAGE);
     gtk_widget_show_all (self.newevent);
     } */

  gtk_notebook_set_current_page (GTK_NOTEBOOK (self.notebook), EVENT_EDIT_PAGE);
  view =
    gtk_notebook_get_nth_page (GTK_NOTEBOOK (self.notebook), EVENT_EDIT_PAGE);
  if (CAL_IS_EVENT_EDIT (view))
    cal_edit_event (CAL_EVENT_EDIT (view), ev);
  gtk_window_set_focus (GTK_WINDOW (self.window),
                        CAL_EVENT_EDIT (view)->entries[EV_SUBJECT]);
}

static void
weekView ()
{
  GtkWidget *view;

  gtk_notebook_set_current_page (GTK_NOTEBOOK (self.notebook), WEEK_VIEW_PAGE);
  view =
    gtk_notebook_get_nth_page (GTK_NOTEBOOK (self.notebook), WEEK_VIEW_PAGE);
  if (CAL_IS_WEEK_VIEW (view))
    cal_week_view_update (CAL_WEEK_VIEW (view));
}


static void
deleteBeforeDate ()
{
  GtkWidget *dialog;

  dialog = cal_go_to_date_new (BEFORE_DATE);

}

static void
gotoDate ()
{
  GtkWidget *dialog;

  dialog = cal_go_to_date_new (GO_TO_DATE);

}

static void
Settings ()
{
  GtkWidget *view;

  gtk_notebook_set_current_page (GTK_NOTEBOOK (self.notebook), SETTING_PAGE);
  view = gtk_notebook_get_nth_page (GTK_NOTEBOOK (self.notebook), SETTING_PAGE);
  if (CAL_IS_SETTING_PAGE (view))
    cal_setting_page_update (CAL_SETTING_PAGE (view));
}



static void
monthView ()
{
  gtk_notebook_set_current_page (GTK_NOTEBOOK (self.notebook), MONTH_VIEW_PAGE);
}



/* Action description table to set up the GtkActionGroup. */
static const GtkActionEntry action_entries[] = {
  {"newEvent", NULL, "_New", "", "", G_CALLBACK (switch_to_new_event)},
  {"dayView", NULL, "_Day View", "", "", G_CALLBACK (switch_to_day_view)},
  {"weekView", NULL, "_Week View", "", "", G_CALLBACK (weekView)},
  {"monthView", NULL, "_Month View", "", "", G_CALLBACK (monthView)},
  {"deleteEvent", NULL, "_Delete"},
  {"gotoDate", NULL, "_Go to date", "", "", G_CALLBACK (gotoDate)},
  {"Settings", NULL, "_Settings", "", "", G_CALLBACK (Settings)},
  {"deleteOne", NULL, "_Delete", "", "",
   G_CALLBACK (cal_day_view_delete_one_event)},
  {"deleteAll", NULL, "_Delete all", "", "",
   G_CALLBACK (cal_month_view_delete_all)},
  {"deleteBeforeDate", NULL, "_Events before date", "", "",
   G_CALLBACK (deleteBeforeDate)}
};

gboolean
create_menus ()
{
  GError *err = NULL;
  gchar *res_path = NULL;
  gchar *ui_file = NULL;

  g_print ("%s :%d\n", __FILE__, __LINE__);
  /* Set up action group and feed UI manager with actions and UI description. */
  self.action_group = gtk_action_group_new ("AllActions");
  self.ui_manager = gtk_ui_manager_new ();

  gtk_action_group_add_actions (self.action_group, action_entries,
                                G_N_ELEMENTS (action_entries), NULL);
  gtk_ui_manager_insert_action_group (self.ui_manager, self.action_group, 0);

  g_print ("%s :%d\n", __FILE__, __LINE__);
  /*Todo: give a default value for user who don't know how to set it. And give a 
     explict explanation for how to set the path. */
  res_path = getenv ("CAL_RES_PATH");
  ui_file = g_build_filename (res_path, "ui-des-calendar.xml", NULL);
  if (!gtk_ui_manager_add_ui_from_file (self.ui_manager, ui_file, &err))
  {
    g_printerr ("Failed to parse menu data: %s\n", err->message);
    g_error_free (err);
    return FALSE;
  }
  g_print ("%s :%d\n", __FILE__, __LINE__);
  g_free (ui_file);
  self.menu_left = gtk_ui_manager_get_widget (self.ui_manager, "/popup-left");

  g_print ("%s :%d\n", __FILE__, __LINE__);
  gpe_skwindow_set_softkey_menu (GPE_SKWINDOW (self.window), SK_LEFT,
                                 self.menu_left);
  gpe_skwindow_set_softkey_menu (GPE_SKWINDOW (self.window), SK_RIGHT, NULL);
  gpe_skwindow_set_softkey_label (GPE_SKWINDOW (self.window), SK_LEFT,
                                  "Options");
  gpe_skwindow_set_softkey_label (GPE_SKWINDOW (self.window), SK_RIGHT, "Back");

  g_signal_connect (self.menu_left, "key-press-event",
                    G_CALLBACK (popup_key_press_cb), NULL);

  g_print ("%s :%d\n", __FILE__, __LINE__);
  return TRUE;
}



int
main (int argc, char *argv[])
{
  time_t nowtime;
  gboolean ret;
  gchar *res_path = NULL;
  gchar *settingfile = NULL;

/*
 * Load the calendar setting file to show the pages  
 */

  res_path = getenv ("CAL_RES_PATH");
  settingfile = g_build_filename (res_path, "setting.cal", NULL);
  self.setting = g_new0 (CalSetting, 1);
  cal_setting_parse_file (self.setting, settingfile);
  g_free (settingfile);


/*
 * Initializing:
 *    open the database; set locale; set viewtime;
 */
  if (!cal_enabler_init ())
  {
    g_print ("calendar enabler init fail!\n");
    return 1;
  }

  gtk_set_locale ();
  gtk_init (&argc, &argv);

  dbus_invoke = FALSE;
  time (&nowtime);
  cal_set_viewtime (nowtime);

/*
 * Create the main window
 */
  self.window = gpe_skwindow_new ();
  gtk_window_set_default_size (GTK_WINDOW (self.window), 320, 240);
  self.num_softkeys =
    gpe_skwindow_get_softkey_count (GPE_SKWINDOW (self.window));
  gtk_window_set_title (GTK_WINDOW (self.window), "calendar main");
  gtk_container_set_border_width (GTK_CONTAINER (self.window), 5);
  g_signal_connect (self.window, "destroy", G_CALLBACK (gtk_main_quit),
                    &self.window);
  g_signal_connect_swapped (self.window, "softkey_activated",
                            G_CALLBACK (softkey_activated_cb), self.window);
  g_print ("%s :%d\n", __FILE__, __LINE__);
/*
 * Added for patched combobox 
 */
  GtkSettings *settings = gtk_widget_get_settings (self.window);
  g_object_set (settings, "gtk-cursor-only-focus", TRUE, NULL);

/*
 * Create the menus for calendar
 */
  ret = create_menus ();
  if (!ret)
  {
    cal_enabler_destroy ();
    return 0;
  }

  g_print ("%s :%d\n", __FILE__, __LINE__);
/*
 * Create notebook
 */
  self.notebook = gtk_notebook_new ();
  gtk_container_add (GTK_CONTAINER (self.window), self.notebook);
  GTK_WIDGET_UNSET_FLAGS (self.notebook, GTK_CAN_FOCUS);
  gtk_notebook_set_show_tabs (GTK_NOTEBOOK (self.notebook), FALSE);
  gtk_notebook_set_scrollable (GTK_NOTEBOOK (self.notebook), TRUE);

  g_print ("%s :%d\n", __FILE__, __LINE__);
/*
 * Add pages to the notebook
 */
  self.monthview = cal_month_view_new ();
  self.dayview = cal_day_view_new ();
  self.newevent = cal_event_edit_new ();
  self.weekview = cal_week_view_new ();
  self.settingpage = cal_setting_page_new ();


  g_print ("%s :%d\n", __FILE__, __LINE__);

  gtk_notebook_append_page (GTK_NOTEBOOK (self.notebook), self.monthview, NULL);
  gtk_notebook_append_page (GTK_NOTEBOOK (self.notebook), self.dayview, NULL);
  gtk_notebook_append_page (GTK_NOTEBOOK (self.notebook), self.weekview, NULL);
  gtk_notebook_append_page (GTK_NOTEBOOK (self.notebook), self.newevent, NULL);
  gtk_notebook_append_page (GTK_NOTEBOOK (self.notebook), self.settingpage,
                            NULL);

  g_signal_connect (GTK_NOTEBOOK (self.notebook), "switch-page",
                    G_CALLBACK (switch_page_cb), NULL);

  printf ("default_view: %d\n", self.setting->default_view);
  gtk_widget_show_all (self.window);

  gtk_notebook_set_current_page (GTK_NOTEBOOK (self.notebook),
                                 self.setting->default_view);
  dbus_init (self.window);

  g_print ("%s :%d\n", __FILE__, __LINE__);
  gtk_main ();

/*
 * Close the database and release the memory allocated after quiting from gtk_main
 */
  cal_enabler_destroy ();
  g_free (self.setting);
  cal_month_view_destroy (CAL_MONTH_VIEW (self.monthview));
  cal_day_view_destroy (CAL_DAY_VIEW (self.dayview));
  cal_week_view_destroy (CAL_WEEK_VIEW (self.weekview));

  return 0;

}

/*
vi:ts=2:nowrap:ai:expandtab
*/

