/*	calendar - calendaring and scheduling application reference implementation
 *
 *  Authors: YANG Lin <lin.yang@orange-ftgroup.com> 
 *  
 *  This software and associated documentation files (the "Software") 
 *  are copyright (C) 2005 LiPS Linux Phone Standards Forum [FranceTelecom] 
 *  All Rights Reserved. 
 *
 *  A copyright license is hereby granted for redistribution and use of 
 *  the Software in source and binary forms, with or without modification, 
 *  provided that the following conditions are met: 
 *  - Redistributions of source code must retain the above copyright notice, 
 *  this copyright license and the following disclaimer. 
 *  - Redistributions in binary form must reproduce the above copyright 
 *   notice, this copyright license and the following disclaimer in the 
 *  documentation and/or other materials provided with the distribution. 
 *  - Neither the name of LiPS  nor the names of its Members may be used 
 *  to endorse or promote products derived from the Software without 
 *  specific prior written permission. 
 *
 *  A patent license for any Necessary Claims owned by Members of LiPS Forum 
 *  to make, have made, use, import, offer to sell, lease and sell or otherwise 
 *  distribute any implementation compliant with the any specification adopted 
 *  by the LiPS Forumcan be obtained from the respective Members on reasonable 
 *  and non-discriminatory terms and conditions and under reciprocity, as 
 *  regulated in more detail in the Internal Policy of the LiPS Forum. 
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDER, ITS MEMBERS AND CONTRIBUTORS 
 *  "AS IS", AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
 *  THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE 
 *  AND NON-INFRINGEMENT ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER, 
 *  ITS MEMBERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
 *  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; 
 *  OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 *  WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 *  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
 *  POSSIBILITY OF SUCH DAMAGE. 
 */


#include <sys/types.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <langinfo.h>

#include <gtk/gtk.h>
#include <gdk/gdkkeysyms.h>
#include <glib.h>
#include <gpe/gpeskwindow.h>
#include <gemwidget/gem.h>
#include <calenabler/cal.h>

#include "monthview.h"
#include "dayview.h"
#include "basicframe.h"
#include "timemanage.h"
#include "calenabler.h"
#include "setting.h"

/*xp : the start of the x direction; xs : the step of the x direction; ys : the step of the y direction*/
static guint xp = 5, xs = 30, ys = 22;
/*nl_item days1[7] =
  { ABDAY_2, ABDAY_3, ABDAY_4, ABDAY_5, ABDAY_6, ABDAY_7, ABDAY_1 };
nl_item days2[7] =
  { ABDAY_1, ABDAY_2, ABDAY_3, ABDAY_4, ABDAY_5, ABDAY_6, ABDAY_7 };*/

gchar *days1[7] = { "Mo", "Tu", "We", "Th", "Fr", "Sa", "Su" };
gchar *days2[7] = { "Su", "Mo", "Tu", "We", "Th", "Fr", "Sa" };

enum
{
  COL_TIME = 0,
  COL_SUM,
  NUM_COLS
};


static void cal_month_view_class_init (CalMonthViewClass * klass);
static void cal_month_view_init (CalMonthView * view);

GType
cal_month_view_get_type (void)
{
  static GType monthview_type = 0;

  if (!monthview_type)
  {
    static const GTypeInfo monthview_info = {
      sizeof (CalMonthViewClass),
      NULL,                     /* base_init */
      NULL,                     /* base_finalize */
      (GClassInitFunc) cal_month_view_class_init,
      NULL,                     /* class_finalize */
      NULL,                     /* class_data */
      sizeof (CalMonthView),
      0,                        /* n_preallocs */
      (GInstanceInitFunc) cal_month_view_init,
    };

    monthview_type =
      g_type_register_static (GTK_TYPE_FRAME, "CalMonthView", &monthview_info,
                              0);
  }

  return monthview_type;
}

static void
cal_month_view_class_init (CalMonthViewClass *klass)
{
  return;
}

static void
title_height_calc (CalMonthView *view)
{
  PangoLayout *pl;
  PangoRectangle pr;
  guint max_height = 0;
  guint i;

  pl = gtk_widget_create_pango_layout (GTK_WIDGET (view->draw), NULL);

  for (i = 0; i < 7; i++)
  {
    /*gchar *s = g_locale_to_utf8 (nl_langinfo (days1[i]), -1,
       NULL, NULL, NULL); */
    gchar *s = g_strdup (days1[i]);
    pango_layout_set_text (pl, s, strlen (s));
    pango_layout_get_pixel_extents (pl, &pr, NULL);

    if (pr.height > max_height)
      max_height = pr.height;

    g_free (s);
  }

  view->title_height = max_height + 8;
#ifdef DEBUG
  printf ("main.c: %s: title_height=%d\n", __FUNCTION__, view->title_height);
#endif
  g_object_unref (pl);
}


static gboolean
month_view_button_press_cb (GtkWidget *widget, GdkEventButton *event,
                            gpointer d)
{
  CalMonthView *view = (CalMonthView *) d;
  guint x = event->x;
  guint y = event->y;
  struct month_render_ctl *c;
  time_t viewtime = cal_get_viewtime ();


  if (x < xp || y < ys)
    return FALSE;

  y -= view->title_height;

  x -= xp;
  x /= xs;
  y /= ys;


  if ((x >= 0) && (x <= 6) && (y >= 0) && (y <= 5))
  {

    c = &(view->rc[x + (y * 7)]);
    if (c->valid)
    {
      if (event->type == GDK_BUTTON_PRESS)
      {

        viewtime = cal_time_from_day (viewtime, c->year, c->month, c->day);
        cal_set_viewtime (viewtime);
        cal_month_view_update (view, BUTTON_PRESS);
        switch_to_day_view ();
      }
    }
  }
  return TRUE;
}


static gboolean
month_view_key_press_cb (GtkWidget *widget, GdkEventKey *k,
                         gpointer user_data)
{
  CalMonthView *view = (CalMonthView *) user_data;
  struct month_render_ctl *c;
  time_t viewtime = cal_get_viewtime ();

  if (k->keyval == GDK_Right)
  {
    if (view->active_day < TOTAL_MONTH_DAYS)
    {
      view->active_day++;

      if (view->rc[view->active_day].year >= 2038)
      {
        GtkWidget *dialog;

        dialog = gtk_message_dialog_new (GTK_WINDOW (self.window),
                                         GTK_DIALOG_MODAL |
                                         GTK_DIALOG_DESTROY_WITH_PARENT,
                                         GTK_MESSAGE_WARNING, GTK_BUTTONS_CLOSE,
                                         "Only display 1976 ~ 2037\n");
        gtk_window_set_decorated (GTK_WINDOW (dialog), TRUE);
        gtk_dialog_run (GTK_DIALOG (dialog));
        gtk_widget_destroy (dialog);

        view->active_day--;
        return TRUE;
      }
      viewtime = cal_time_from_day (viewtime, view->rc[view->active_day].year,
                                    view->rc[view->active_day].month,
                                    view->rc[view->active_day].day);
      cal_set_viewtime (viewtime);

      cal_month_view_update (view, KEY_PRESS);
    }
    return TRUE;
  }

  if (k->keyval == GDK_Left)
  {
    if (view->active_day)
    {
      view->active_day--;

      viewtime = cal_time_from_day (viewtime, view->rc[view->active_day].year,
                                    view->rc[view->active_day].month,
                                    view->rc[view->active_day].day);
      if (viewtime <= 0)
      {
        GtkWidget *dialog;

        dialog = gtk_message_dialog_new (GTK_WINDOW (self.window),
                                         GTK_DIALOG_MODAL |
                                         GTK_DIALOG_DESTROY_WITH_PARENT,
                                         GTK_MESSAGE_WARNING, GTK_BUTTONS_CLOSE,
                                         "Only display 1976 ~ 2037\n");
        gtk_window_set_decorated (GTK_WINDOW (dialog), TRUE);
        gtk_dialog_run (GTK_DIALOG (dialog));
        gtk_widget_destroy (dialog);

        view->active_day++;
        return TRUE;
      }
      cal_set_viewtime (viewtime);
      cal_month_view_update (view, KEY_PRESS);
    }
    return TRUE;
  }

  if (k->keyval == GDK_Down)
  {
    /*still in the same page*/
    if (view->active_day <= TOTAL_MONTH_DAYS - 7)       
    {
      view->active_day += 7;
      /*printf ("%s:view->active_day,year=%d, month=%d, day=%d\n",
         __FUNCTION__, view->rc[view->active_day].year,
         view->rc[view->active_day].month,
         view->rc[view->active_day].day); */

      if (view->rc[view->active_day].year > 2037)
      {
        GtkWidget *dialog;

        dialog = gtk_message_dialog_new (GTK_WINDOW (self.window),
                                         GTK_DIALOG_MODAL |
                                         GTK_DIALOG_DESTROY_WITH_PARENT,
                                         GTK_MESSAGE_WARNING, GTK_BUTTONS_CLOSE,
                                         "Only display 1976.1.1 ~ 2037.12.31\n");
        gtk_window_set_decorated (GTK_WINDOW (dialog), TRUE);
        gtk_dialog_run (GTK_DIALOG (dialog));
        gtk_widget_destroy (dialog);

        view->active_day -= 7;
        return TRUE;
      }

      viewtime = cal_time_from_day (viewtime, view->rc[view->active_day].year,
                                    view->rc[view->active_day].month,
                                    view->rc[view->active_day].day);
#ifdef DEBUG
      /*printf ("%s:viewtime=%d\n", __FUNCTION__, viewtime);*/
#endif
    }
    else
    {
      struct tm tm_copy;

      localtime_r (&viewtime, &tm_copy);
#ifdef DEBUG
      /*printf ("%s:tm_copy.tm_mday=%d, tm_mon=%d, tm_year=%d\n",__FUNCTION__, 
              tm_copy.tm_mday, tm_copy.tm_mon, tm_copy.tm_year);*/
#endif
      tm_copy.tm_mday += 7;


      if ((tm_copy.tm_year > 137) ||
          ((tm_copy.tm_year == 137) &&
           (tm_copy.tm_mon == 11) && (tm_copy.tm_mday >= 31)))
      {
        GtkWidget *dialog;

        dialog = gtk_message_dialog_new (GTK_WINDOW (self.window),
                                         GTK_DIALOG_MODAL |
                                         GTK_DIALOG_DESTROY_WITH_PARENT,
                                         GTK_MESSAGE_WARNING, GTK_BUTTONS_CLOSE,
                                         "Only display 1976 ~ 2037\n");
        gtk_window_set_decorated (GTK_WINDOW (dialog), TRUE);
        gtk_dialog_run (GTK_DIALOG (dialog));
        gtk_widget_destroy (dialog);
        return TRUE;
      }
      viewtime = mktime (&tm_copy);
      view->active_day = view->active_day + 7 - TOTAL_MONTH_DAYS;
    }

    cal_set_viewtime (viewtime);
    cal_month_view_update (view, KEY_PRESS);
    return TRUE;
  }

  if (k->keyval == GDK_Up)
  {
    /*still in the same page*/
    if (view->active_day > 7)   
    {
      view->active_day -= 7;
      viewtime = cal_time_from_day (viewtime, view->rc[view->active_day].year,
                                    view->rc[view->active_day].month,
                                    view->rc[view->active_day].day);
      if (viewtime <= 0)
      {
        GtkWidget *dialog;

        dialog = gtk_message_dialog_new (GTK_WINDOW (self.window),
                                         GTK_DIALOG_MODAL |
                                         GTK_DIALOG_DESTROY_WITH_PARENT,
                                         GTK_MESSAGE_WARNING, GTK_BUTTONS_CLOSE,
                                         "Only display 1976 ~ 2037\n");
        gtk_window_set_decorated (GTK_WINDOW (dialog), TRUE);
        gtk_dialog_run (GTK_DIALOG (dialog));
        gtk_widget_destroy (dialog);
        view->active_day += 7;
        return TRUE;
      }
    }
    else                        /*switch to the previous page*/
    {
      struct tm tm_copy;

      localtime_r (&viewtime, &tm_copy);
      tm_copy.tm_mday -= 7;
      viewtime = mktime (&tm_copy);
      if (viewtime <= 0)
      {
        GtkWidget *dialog;

        dialog = gtk_message_dialog_new (GTK_WINDOW (self.window),
                                         GTK_DIALOG_MODAL |
                                         GTK_DIALOG_DESTROY_WITH_PARENT,
                                         GTK_MESSAGE_WARNING, GTK_BUTTONS_CLOSE,
                                         "Only display 1976 ~ 2037\n");
        gtk_window_set_decorated (GTK_WINDOW (dialog), TRUE);
        gtk_dialog_run (GTK_DIALOG (dialog));
        gtk_widget_destroy (dialog);

        return TRUE;
      }
      view->active_day = view->active_day + TOTAL_MONTH_DAYS - 7;
    }

    cal_set_viewtime (viewtime);
    cal_month_view_update (view, KEY_PRESS);
    return TRUE;
  }


  if (k->keyval == GDK_Return)
  {
    c = &(view->rc[view->active_day]);
    if (c->valid)
    {
      struct tm tm;
      time_t selected_time;
      localtime_r (&viewtime, &tm);
      tm.tm_year = c->year - 1900;
      tm.tm_mon = c->month - 1;
      tm.tm_mday = c->day;
      tm.tm_hour = 0;
      tm.tm_min = 0;
      tm.tm_sec = 0;
      selected_time = mktime (&tm);

      cal_set_viewtime (selected_time);
      switch_to_day_view ();

    }
    return TRUE;
  }

  return FALSE;
}

static gint
month_view_draw_expose_cb (GtkWidget * widget,
                           GdkEventExpose * event, gpointer user_data)
{
  CalMonthView *view = (CalMonthView *) user_data;
  GtkDrawingArea *darea;
  GdkDrawable *drawable;
  guint width, height;

  GdkGC *black_gc;
  GdkGC *gray_gc;
  GdkGC *white_gc;
  GdkGC *cream_gc;
  GdkGC *light_gray_gc;
  GdkGC *yellow_gc;
  GdkGC *salmon_gc;
  GdkGC *blue1_gc;              /*light blue, for the normal day */
  GdkGC *blue2_gc;              /*dark blue, for today */
  GdkGC *blue_gc;

  GdkColor cream;
  GdkColor light_gray;
  GdkColor yellow;
  GdkColor salmon;
  GdkColor blue;
  GdkColor blue1;
  GdkColor blue2;

  GdkColormap *colormap;
  guint i, j;
  PangoLayout *pl;
  PangoLayout *pl_evt;
  PangoRectangle pr;
  /*nl_item *days;*/
  gchar **days;
  g_return_val_if_fail (widget != NULL, TRUE);
  g_return_val_if_fail (GTK_IS_DRAWING_AREA (widget), TRUE);
#ifdef DEBUG
  printf ("entering %s\n", __FUNCTION__);
#endif
  days = (self.setting->week_starts_monday) ? days1 : days2;

  colormap = gdk_window_get_colormap (widget->window);

  cream_gc = gdk_gc_new (widget->window);
  gdk_gc_copy (cream_gc, widget->style->black_gc);
  cream.red = 65535;
  cream.green = 60138;
  cream.blue = 60138;
  gdk_colormap_alloc_color (colormap, &cream, FALSE, TRUE);
  gdk_gc_set_foreground (cream_gc, &cream);

  light_gray_gc = gdk_gc_new (widget->window);
  gdk_gc_copy (light_gray_gc, widget->style->black_gc);
  light_gray.red = 53040;
  light_gray.green = 53040;
  light_gray.blue = 53040;
  gdk_colormap_alloc_color (colormap, &light_gray, FALSE, TRUE);
  gdk_gc_set_foreground (light_gray_gc, &light_gray);

  blue_gc = gdk_gc_new (widget->window);
  gdk_gc_copy (blue_gc, widget->style->black_gc);
  blue.red = 14906;
  blue.green = 35980;
  blue.blue = 55769;
  gdk_colormap_alloc_color (colormap, &blue, FALSE, TRUE);
  gdk_gc_set_foreground (blue_gc, &blue);

  blue1_gc = gdk_gc_new (widget->window);
  gdk_gc_copy (blue1_gc, widget->style->black_gc);
  blue1.red = 53199;
  blue1.green = 59110;
  blue1.blue = 65535;
  gdk_colormap_alloc_color (colormap, &blue1, FALSE, TRUE);
  gdk_gc_set_foreground (blue1_gc, &blue1);

  blue2_gc = gdk_gc_new (widget->window);
  gdk_gc_copy (blue2_gc, widget->style->black_gc);
  blue2.red = 36237;
  blue2.green = 47802;
  blue2.blue = 58596;
  gdk_colormap_alloc_color (colormap, &blue2, FALSE, TRUE);
  gdk_gc_set_foreground (blue2_gc, &blue2);

  yellow_gc = gdk_gc_new (widget->window);
  gdk_gc_copy (yellow_gc, widget->style->black_gc);
  gdk_color_parse ("palegoldenrod", &yellow);
  gdk_colormap_alloc_color (colormap, &yellow, FALSE, TRUE);
  gdk_gc_set_foreground (yellow_gc, &yellow);

  salmon_gc = gdk_gc_new (widget->window);
  gdk_gc_copy (salmon_gc, widget->style->black_gc);
  gdk_color_parse ("light salmon", &salmon);
  gdk_colormap_alloc_color (colormap, &salmon, FALSE, TRUE);
  gdk_gc_set_foreground (salmon_gc, &salmon);


  white_gc = widget->style->white_gc;
  gray_gc = widget->style->bg_gc[GTK_STATE_NORMAL];
  black_gc = widget->style->black_gc;

  gdk_gc_set_clip_rectangle (black_gc, &event->area);
  gdk_gc_set_clip_rectangle (gray_gc, &event->area);
  gdk_gc_set_clip_rectangle (white_gc, &event->area);
  gdk_gc_set_clip_rectangle (cream_gc, &event->area);
  gdk_gc_set_clip_rectangle (light_gray_gc, &event->area);
  gdk_gc_set_clip_rectangle (yellow_gc, &event->area);
  gdk_gc_set_clip_rectangle (salmon_gc, &event->area);
  gdk_gc_set_clip_rectangle (blue_gc, &event->area);
  gdk_gc_set_clip_rectangle (blue1_gc, &event->area);
  gdk_gc_set_clip_rectangle (blue2_gc, &event->area);

  pl = gtk_widget_create_pango_layout (GTK_WIDGET (widget), NULL);
  pl_evt = gtk_widget_create_pango_layout (GTK_WIDGET (widget), NULL);

  darea = GTK_DRAWING_AREA (widget);
  drawable = widget->window;

  width = widget->allocation.width;
  height = widget->allocation.height;

  gdk_window_clear_area (drawable,
                         event->area.x, event->area.y,
                         event->area.width, event->area.height);


  for (i = 0; i < 7; i++)
  {
    guint x = xp + (i * xs);
    guint w;
    /*gchar *s = g_locale_to_utf8 (nl_langinfo (days[i]), -1,
       NULL, NULL, NULL); */
    gchar *s = g_strdup (days[i]);
    pango_layout_set_text (pl, s, strlen (s));
    pango_layout_get_pixel_extents (pl, &pr, NULL);
    w = pr.width;

    if ((i > 5) || (i < 1))
      gdk_draw_rectangle (drawable, cream_gc, TRUE, x, 0, xs, ys);
    else
      gdk_draw_rectangle (drawable, blue1_gc, TRUE, x, 0, xs, ys);

    /*Write the week day in title part */
    gtk_paint_layout (widget->style,
                      widget->window,
                      GTK_WIDGET_STATE (widget),
                      FALSE,
                      &event->area, widget, "label", x + (xs - w) / 2, 2, pl);


    for (j = 0; j < (TOTAL_MONTH_DAYS / 7); j++)
    {
      gchar *buffer;
      guint d = i + (7 * j);
      struct month_render_ctl *c = &(view->rc[d]);
      guint y = (j + 1) * ys;
      guint w;
      char buf[10];

      if (c->valid)
      {

        gdk_draw_rectangle (drawable, blue1_gc, TRUE, x, y, xs, ys);

        if (c->today)
          gdk_draw_rectangle (drawable, blue2_gc, TRUE, x, y, xs, ys);


        if (c->events)
          gtk_paint_arrow (widget->style,
                           widget->window,
                           GTK_WIDGET_STATE (widget),
                           GTK_SHADOW_IN,
                           &event->area,
                           widget,
                           "label",
                           GTK_ARROW_RIGHT,
                           TRUE, x + xs - 10, y + ys - 10, 10, 10);


      }
      else
      {
        gdk_draw_rectangle (drawable, gray_gc, TRUE,
                            x + 1, y + 1, xs - 1, ys - 1);

        if (i == 0)
          gdk_draw_line (drawable, light_gray_gc, x + 1, y + 1, x + 1, y + ys);
        gdk_draw_line (drawable, light_gray_gc, x + 1, y + ys, x + xs, y + ys);
        gdk_draw_line (drawable, light_gray_gc, x + xs, y + 1, x + xs, y + ys);
      }

      snprintf (buf, sizeof (buf), "%d", c->day);
      buffer = g_locale_to_utf8 (buf, -1, NULL, NULL, NULL);
      pango_layout_set_text (pl, buffer, strlen (buffer));
      pango_layout_get_pixel_extents (pl, &pr, NULL);
      w = pr.width;
      gtk_paint_layout (widget->style,
                        widget->window,
                        GTK_WIDGET_STATE (widget),
                        FALSE, &event->area, widget, "label", x + 2, y, pl);
      g_free (buffer);
    }

  }

  for (i = 0; i < 7; i++)
  {
    guint x = xp + (i * xs);

    for (j = 0; j < (TOTAL_MONTH_DAYS / 7); j++)
    {
      guint y = j * ys;
      gdk_draw_rectangle (drawable, light_gray_gc, FALSE, x, y, xs, ys);
    }
  }

  for (i = 0; i < 7; i++)
  {
    guint x = xp + (i * xs);

    for (j = 0; j < (TOTAL_MONTH_DAYS / 7); j++)
    {
      guint d = i + (7 * j);
      struct month_render_ctl *c = &(view->rc[d]);
      guint y = (j + 1) * ys;

      if (!(self.setting->force_today) && c->active)
      {
        gdk_draw_rectangle (drawable, blue_gc, FALSE, x, y, xs, ys);
        gdk_draw_rectangle (drawable, blue_gc, FALSE,
                            x + 1, y + 1, xs - 2, ys - 2);


      }

      if (!(self.setting->force_today) && c->active)
      {

        if (c->events)
        {
          cal_event_t *ev;
          gchar *string = NULL;
          gchar *description;
          time_t start;
          GSList *iter;
          char starttime[20];


          for (iter = c->events; iter; iter = iter->next)
          {
            struct tm tm;

            ev = iter->data;
            description = event_get_description (ev);
            start = event_get_start (ev);
            localtime_r (&start, &tm);

            strftime (starttime, sizeof (starttime), "%R", &tm);

            if (string != NULL)
              string =
                g_strconcat (string, starttime, "  ", description, "\n", NULL);
            else
              string = g_strconcat (starttime, "  ", description, "\n", NULL);


          }
          gtk_label_set_text (GTK_LABEL (view->labeltime1), string);
          g_free (description);
          g_free (string);
        }
        else                    //if (c->events)
          gtk_label_set_text (GTK_LABEL (view->labeltime1), "No events");
      }
    }
  }

  gdk_gc_set_clip_rectangle (black_gc, NULL);
  gdk_gc_set_clip_rectangle (gray_gc, NULL);
  gdk_gc_set_clip_rectangle (white_gc, NULL);

  gdk_gc_unref (blue_gc);
  gdk_gc_unref (blue1_gc);
  gdk_gc_unref (blue2_gc);
  gdk_gc_unref (cream_gc);
  gdk_gc_unref (light_gray_gc);
  gdk_gc_unref (yellow_gc);
  gdk_gc_unref (salmon_gc);


  g_object_unref (pl);

  return TRUE;
}



gboolean
month_view_day_indicate_update (CalMonthView *view)
{
  gchar buffer[255];
  struct tm tm;
  time_t viewtime = cal_get_viewtime ();

  localtime_r (&viewtime, &tm);
  strftime (buffer, sizeof (buffer), MONTH_TIME_FMT, &tm);
  gtk_label_set_text (GTK_LABEL (view->date_indicate), buffer);

  return TRUE;
}

void
month_view_day_events_update (CalMonthView *view)
{
  time_t viewtime = cal_get_viewtime ();
  struct tm tm_start, tm_end;
  time_t start, end;
  guint day;

  g_assert (CAL_IS_MONTH_VIEW (view));
  localtime_r (&viewtime, &tm_start);
  localtime_r (&viewtime, &tm_end);

  for (day = 1; day <= view->monthdays; day++)
  {

    tm_start.tm_mday = day;
    tm_start.tm_hour = 0;
    tm_start.tm_min = 0;
    tm_start.tm_sec = 0;
    start = mktime (&tm_start);

    tm_end.tm_mday = day;
    tm_end.tm_hour = 23;
    tm_end.tm_min = 59;
    tm_end.tm_sec = 59;
    end = mktime (&tm_end);

    /*if (!tm_start.tm_isdst) 
       {
       start+=60*60;
       end+=60*60;
       } */

    if (view->day_events[day])
      cal_event_list_destroy (view->day_events[day]);
    view->day_events[day] = cal_event_list_init (start, end);
    /*if (view->day_events[day])
       {

       printf ("%s; day_events [%d] has events\n", __FUNCTION__, day);
       //cal_event_list_print (view->day_events[day]);
       } */
  }
}



gboolean
cal_month_view_update (CalMonthView *view, guint flag)
{
  guint day, year, month;
  gint wday;
  time_t lastviewtime = cal_get_lastviewtime ();
  time_t viewtime = cal_get_viewtime ();
  struct tm tm_view, tm_lastview;
  gboolean updateall = FALSE;

  /*printf ("entering %s\n", __FUNCTION__);*/

/*Calculate the real year, month and day for each grid in the month view, and store them into rc[ ]  */
  localtime_r (&viewtime, &tm_view);
  localtime_r (&lastviewtime, &tm_lastview);


  if ((flag == OTHER) ||
      ((flag == KEY_PRESS) &&
       !((tm_lastview.tm_year == tm_view.tm_year)
         && (tm_lastview.tm_mon == tm_view.tm_mon))))
  {
    updateall = TRUE;
    /*printf ("%s:updateall = TRUE\n", __FUNCTION__);*/
  }
  else
  {
    updateall = FALSE;
    /*printf ("%s:updateall = FALSE\n", __FUNCTION__);*/
  }


  if (updateall)
  {

    year = tm_view.tm_year + 1900;
    month = tm_view.tm_mon + 1;

    view->monthdays = cal_days_in_month (year, month - 1);
    if (month >= 2)
      view->premonthdays = cal_days_in_month (year, month - 2);
    else
      view->premonthdays = cal_days_in_month (year - 1, month + 10);

    /*printf ("%s:view->premonthdays=%d\n", __FUNCTION__, view->premonthdays);*/
    /*Get the day_events[ ] values */
    month_view_day_events_update (view);

    wday = 1 - cal_day_of_week (year, month, 1);

    for (day = 0; day < TOTAL_MONTH_DAYS; day++)
    {
      gint rday = day + wday - self.setting->week_offset;
      struct month_render_ctl *c = &(view->rc[day]);

      if ((rday > 0) && (rday <= view->monthdays))
      {


        c->valid = TRUE;

        c->day = rday;
        c->month = month;
        c->year = year;

        if ((c->day == view->tm_today.tm_mday)
            && (c->month == view->tm_today.tm_mon + 1)
            && (c->year == view->tm_today.tm_year + 1900))
          c->today = TRUE;
        else
          c->today = FALSE;


        c->events = view->day_events[c->day];

        /*if (c->events)
           printf ("view->day_events[%d], c[%d] has events\n", c->day, c->day); */
      }
      else
      {
        c->valid = FALSE;
        c->today = FALSE;
        c->events = NULL;

        if (rday <= 0)
        {
          /*printf ("c->day =%d, c->month=%d, c->year=%d\n", c->day,c->month, c->year);*/
          c->day = rday + view->premonthdays;
          c->month = month - 1;
          if (c->month > 0)
            c->year = year;
          else
          {
            c->month += 12;
            c->year--;
          }
        }
        else if (rday > view->monthdays)
        {

          c->day = rday - view->monthdays;
          /*printf ("%s:year=%d,month=%d,day=%d\n",__FUNCTION__,
             year, month, c->day);  */
          c->month = month + 1;
          if (c->month <= 12)
            c->year = year;
          else
          {

            c->month -= 12;

            c->year = year + 1;
            /*printf ("%s:c->year=%d,c->month=%d,c->day=%d\n",__FUNCTION__,
               c->year, c->month, c->day);  */

          }
        }
      }


      if (rday == tm_view.tm_mday)
      {
        c->active = TRUE;
        view->active_day = day;
      }
      else
        c->active = FALSE;

    }
  }
  else                          /*only update active day info */
  {
    for (day = 0; day < TOTAL_MONTH_DAYS; day++)
    {
      struct month_render_ctl *c = &(view->rc[day]);

      if ((c->day == tm_view.tm_mday) && (c->month == tm_view.tm_mon + 1))
      {
        c->active = TRUE;
        view->active_day = day;
      }
      else
        c->active = FALSE;
    }
  }

  gtk_widget_queue_draw (view->draw);
  month_view_day_indicate_update (view);

  return TRUE;
}


gboolean
cal_month_view_delete_month_events (CalMonthView *view)
{
  guint day;
  cal_eventbook_t *book = cal_enabler_get_book ();
  gboolean flag = FALSE;

  for (day = 0; day < 32; day++)
  {
    GSList *iter;
    for (iter = view->day_events[day]; iter != NULL; iter = iter->next)
    {
      cal_event_t *ev = iter->data;
      gchar *uid;
      gint ret;

      flag = TRUE;
      uid = event_get_uid (ev);
      ret = eventbook_remove_event (book, uid);
      /*printf ("%s: eventbook_remove_event: %d\n", __FUNCTION__, ret);*/

      g_free (uid);
    }
  }
  return flag;
}


static gboolean
_cal_month_view_delete_all_events (CalMonthView *view)
{
  cal_eventbook_t *book = cal_enabler_get_book ();

  eventbook_remove_all_evnts (book);

  return TRUE;
}

void
delete_all_softkey_activated_cb (GtkWidget *widget, gint key_number)
{
  GtkNotebook *notebook;
  GtkWidget *view;

  if (key_number == SK_LEFT)
  {
    notebook = (GtkNotebook *) self.notebook;
    view = gtk_notebook_get_nth_page (GTK_NOTEBOOK (notebook), MONTH_VIEW_PAGE);
    if (CAL_IS_MONTH_VIEW (view))
      if (_cal_month_view_delete_all_events (CAL_MONTH_VIEW (view)))
        update_all_views ();
  }
  gtk_widget_destroy (widget);
}

void
cal_month_view_delete_all (void)
{
  GtkWidget *dialog;

  dialog = gpe_skwindow_new ();

  gem_sk_window_set_delete_warning (dialog, "Delete all events?");
  gem_sk_dialog_set_position (GTK_WINDOW (self.window), GPE_SKWINDOW (dialog),
                              0);

  g_signal_connect_swapped (dialog, "softkey_activated",
                            G_CALLBACK (delete_all_softkey_activated_cb),
                            dialog);

  gtk_widget_show_all (dialog);
}




static void
cal_month_view_init (CalMonthView *view)
{
  guint day;
  time_t viewtime = cal_get_viewtime ();
  struct tm tm;

  localtime_r (&viewtime, &tm);

  view->tm_today = tm;

/*Initialize the array rc[ ] and day_events[ ]*/
  for (day = 0; day < TOTAL_MONTH_DAYS; day++)
  {
    view->rc[day].valid = FALSE;
    view->rc[day].today = FALSE;
    view->rc[day].active = FALSE;
    view->rc[day].events = NULL;
  }

  for (day = 0; day < 32; day++)
    view->day_events[day] = NULL;

/*Set up the widget*/
  view->vbox = gtk_vbox_new (FALSE, 0);

  view->date_indicate = gtk_label_new ("day indicate");


  gtk_box_pack_start (GTK_BOX (view->vbox), view->date_indicate, FALSE, FALSE,
                      0);
  view->draw = gtk_drawing_area_new ();

  gtk_box_pack_start (GTK_BOX (view->vbox), view->draw, TRUE, TRUE, 0);

  view->frame_evindicate = gtk_frame_new (NULL);
  gtk_box_pack_start (GTK_BOX (view->vbox), view->frame_evindicate, TRUE, TRUE,
                      0);
  gtk_widget_set_size_request (view->draw, 210, 154);
  gtk_widget_set_size_request (view->frame_evindicate, 210, 60);
  view->labeltime1 = gtk_label_new ("TIME");
  gtk_misc_set_alignment (GTK_MISC (view->labeltime1), 0, 0);
  gtk_container_add (GTK_CONTAINER (view->frame_evindicate), view->labeltime1);


  g_signal_connect (G_OBJECT (view->draw), "expose_event",
                    G_CALLBACK (month_view_draw_expose_cb), view);

  g_signal_connect (G_OBJECT (view->draw), "button-press-event",
                    G_CALLBACK (month_view_button_press_cb), view);

  gtk_widget_add_events (GTK_WIDGET (view->draw),
                         GDK_BUTTON_PRESS_MASK | GDK_BUTTON_RELEASE_MASK);

  GTK_WIDGET_SET_FLAGS (view->draw, GTK_CAN_FOCUS);
  /*gtk_widget_grab_focus(view->draw);  */
  g_signal_connect (G_OBJECT (view->draw), "key_press_event",
                    G_CALLBACK (month_view_key_press_cb), view);

  /*gtk_widget_add_events (GTK_WIDGET (view->draw),
     GDK_KEY_PRESS_MASK | GDK_KEY_RELEASE_MASK); */





/*Calculate the title_height*/
  title_height_calc (view);


  cal_month_view_update (view, OTHER);

  gtk_container_add (GTK_CONTAINER (view), GTK_WIDGET (view->vbox));
  return;
}

GtkWidget *
cal_month_view_new (void)
{
  return gtk_widget_new (CAL_TYPE_MONTH_VIEW, NULL);
}

void
cal_month_view_destroy (CalMonthView *view)
{
  guint day;
  struct tm tm_start;
  guint days;
  guint year, month;
  struct tm today;
  time_t t, viewtime = cal_get_viewtime ();

  time (&t);
  localtime_r (&t, &today);
  localtime_r (&viewtime, &tm_start);
  year = tm_start.tm_year + 1900;
  month = tm_start.tm_mon;
  days = cal_days_in_month (year, month);
  for (day = 0; day <= days; day++)
    if (view->day_events[day])
    {
      cal_event_list_destroy (view->day_events[day]);
      view->day_events[day] = NULL;
    }
}

/*
vi:ts=2:nowrap:ai:expandtab
*/

