/*	calendar - calendaring and scheduling application reference implementation
 *
 *  Authors: YANG Lin <lin.yang@orange-ftgroup.com> 
 *  
 *  This software and associated documentation files (the "Software") 
 *  are copyright (C) 2005 LiPS Linux Phone Standards Forum [FranceTelecom] 
 *  All Rights Reserved. 
 *
 *  A copyright license is hereby granted for redistribution and use of 
 *  the Software in source and binary forms, with or without modification, 
 *  provided that the following conditions are met: 
 *  - Redistributions of source code must retain the above copyright notice, 
 *  this copyright license and the following disclaimer. 
 *  - Redistributions in binary form must reproduce the above copyright 
 *   notice, this copyright license and the following disclaimer in the 
 *  documentation and/or other materials provided with the distribution. 
 *  - Neither the name of LiPS  nor the names of its Members may be used 
 *  to endorse or promote products derived from the Software without 
 *  specific prior written permission. 
 *
 *  A patent license for any Necessary Claims owned by Members of LiPS Forum 
 *  to make, have made, use, import, offer to sell, lease and sell or otherwise 
 *  distribute any implementation compliant with the any specification adopted 
 *  by the LiPS Forumcan be obtained from the respective Members on reasonable 
 *  and non-discriminatory terms and conditions and under reciprocity, as 
 *  regulated in more detail in the Internal Policy of the LiPS Forum. 
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDER, ITS MEMBERS AND CONTRIBUTORS 
 *  "AS IS", AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
 *  THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE 
 *  AND NON-INFRINGEMENT ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER, 
 *  ITS MEMBERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
 *  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; 
 *  OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 *  WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 *  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
 *  POSSIBILITY OF SUCH DAMAGE. 
 */


#ifndef _MONTH_VIEW_H_
#define _MONTH_VIEW_H_

#include <gtk/gtk.h>
#include "timemanage.h"

#ifdef __cplusplus
extern "C"
{
#endif

#define  CAL_TYPE_MONTH_VIEW                  (cal_month_view_get_type ())
#define  CAL_MONTH_VIEW(obj)                  (G_TYPE_CHECK_INSTANCE_CAST ((obj), CAL_TYPE_MONTH_VIEW, CalMonthView))
#define  CAL_MONTH_VIEW_CLASS(klass)          (G_TYPE_CHECK_CLASS_CAST ((klass), CAL_TYPE_MONTH_VIEW, CalMonthViewClass))
#define  CAL_IS_MONTH_VIEW(obj)               (G_TYPE_CHECK_INSTANCE_TYPE ((obj), CAL_TYPE_MONTH_VIEW))
#define  CAL_IS_MONTH_VIEW_CLASS(klass)       (G_TYPE_CHECK_CLASS_TYPE ((klass), CAL_TYPE_MONTH_VIEW))
#define  CAL_MONTH_VIEW_GET_CLASS(obj)        (G_TYPE_INSTANCE_GET_CLASS ((obj), CAL_TYPE_MONTH_VIEW, CalMonthViewClass))

  typedef struct _CalMonthView CalMonthView;
  typedef struct _CalMonthViewClass CalMonthViewClass;

/*
* month view page switch:
*      1.go to date
*      2."key_press" event received and focus move to other month.
*
* This structure is used to hold all infor of one cell in the month view.
*   day, month, year: hold the date infor of the cell. These need to be update
*                      when month view page switch;
*    valid:            indicate whether this cell is in current month. This need
*                      to be update when month view page switch;
*    tody:              indicate whether this cell is current day. This need to be 
*                      update when month view page switch;
*    active:            indicate the position of the focus. This need to be update
*                      when "key_press" or "button_press" or any other update event
*    events:            indicate the events happen in this cell's date. This need 
*                      to be update when month view page switch or add new events.
*/
  struct month_render_ctl
  {
    gint day, month, year;
    gboolean valid;
    gboolean today;
    gboolean active;
    GSList *events;
  };

/**The array rc[ ] is used to store the info of each day including: year,month,day, events in the day.
*The array day_events[ ] is to store the events for each day
*/
  struct _CalMonthView
  {
    GtkFrame frame;

    GtkWidget *vbox;
    GtkWidget *date_indicate;
    GtkWidget *draw;
    GtkWidget *vbox1;
    GtkWidget *frame_evindicate;
    GtkWidget *labeltime1;


    struct month_render_ctl rc[TOTAL_MONTH_DAYS];
    GSList *day_events[32];
    guint monthdays;            /*days in this month*/
    guint premonthdays;         /*days in the last month*/

    guint title_height;
    guint active_day;
    struct tm tm_today;         /*today is initialized when month view init*/

  };

  struct _CalMonthViewClass
  {
    GtkFrameClass parent_class;
  };



  GType cal_month_view_get_type (void) G_GNUC_CONST;
  GtkWidget *cal_month_view_new (void);

  enum
  {
    KEY_PRESS = 0,
    BUTTON_PRESS,
    OTHER
  };
  
/*
 *"flag" indicate which kind of update need to be done
 *  1.flag = KEY_PRESS
 *    If focus is still in the same month,only need to update active day info.
 *    If focus is in other month,need to update all
 *  2.flag = BUTTON_PRESS
 *    only need to update active day info
 *  3.flag = OTHER (initiate/go to date/add event/delete event)
 *     all need to be update
*/
  gboolean cal_month_view_update (CalMonthView *view, guint flag);
  void cal_month_view_delete_all (void);
  gboolean cal_month_view_delete_month_events (CalMonthView *view);
  void cal_month_view_destroy (CalMonthView *view);
#ifdef __cplusplus
}
#endif

#endif /*_MONTH_VIEW_H_*/
/*
vi:ts=2:nowrap:ai:expandtab
*/

