/*	calendar - calendaring and scheduling application reference implementation
 *
 *  Authors: YANG Lin <lin.yang@orange-ftgroup.com> 
 *  
 *  This software and associated documentation files (the "Software") 
 *  are copyright (C) 2005 LiPS Linux Phone Standards Forum [FranceTelecom] 
 *  All Rights Reserved. 
 *
 *  A copyright license is hereby granted for redistribution and use of 
 *  the Software in source and binary forms, with or without modification, 
 *  provided that the following conditions are met: 
 *  - Redistributions of source code must retain the above copyright notice, 
 *  this copyright license and the following disclaimer. 
 *  - Redistributions in binary form must reproduce the above copyright 
 *   notice, this copyright license and the following disclaimer in the 
 *  documentation and/or other materials provided with the distribution. 
 *  - Neither the name of LiPS  nor the names of its Members may be used 
 *  to endorse or promote products derived from the Software without 
 *  specific prior written permission. 
 *
 *  A patent license for any Necessary Claims owned by Members of LiPS Forum 
 *  to make, have made, use, import, offer to sell, lease and sell or otherwise 
 *  distribute any implementation compliant with the any specification adopted 
 *  by the LiPS Forumcan be obtained from the respective Members on reasonable 
 *  and non-discriminatory terms and conditions and under reciprocity, as 
 *  regulated in more detail in the Internal Policy of the LiPS Forum. 
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDER, ITS MEMBERS AND CONTRIBUTORS 
 *  "AS IS", AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
 *  THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE 
 *  AND NON-INFRINGEMENT ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER, 
 *  ITS MEMBERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
 *  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; 
 *  OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 *  WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 *  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
 *  POSSIBILITY OF SUCH DAMAGE. 
 */


#include <stdio.h>
#include <stdlib.h>
#include <glib.h>
#include <glib/gprintf.h>
#include <gemwidget/gtkautoscrolledwindow.h>
#include <gpe/gpeskwindow.h>


#include "basicframe.h"

static void cal_setting_page_class_init (CalSettingPageClass *klass);
static void cal_setting_page_init (CalSettingPage *view);

enum
{
  WEEK_STARTS_MONDAY = 0,
  DEFAULT_VIEW = 1,
  FORCE_TODAY = 2
};

GType
cal_setting_page_get_type (void)
{
  static GType settingpage_type = 0;

  if (!settingpage_type)
  {
    static const GTypeInfo settingpage_info = {
      sizeof (CalSettingPageClass),
      NULL,                     /* base_init */
      NULL,                     /* base_finalize */
      (GClassInitFunc) cal_setting_page_class_init,
      NULL,                     /* class_finalize */
      NULL,                     /* class_data */
      sizeof (CalSettingPage),
      0,                        /* n_preallocs */
      (GInstanceInitFunc) cal_setting_page_init,
    };

    settingpage_type =
      g_type_register_static (GTK_TYPE_AUTO_SCROLLED_WINDOW, "CalSettingPage",
                              &settingpage_info, 0);
  }

  return settingpage_type;
}

static void
cal_setting_page_class_init (CalSettingPageClass *klass)
{
  return;
}


gboolean
cal_setting_parse_file (CalSetting *setting, const gchar *fname)
{
  GIOChannel *io = NULL;
  GIOStatus status = G_IO_STATUS_NORMAL;


  g_assert (setting);
  g_assert (fname);
#ifdef DEBUG
  printf ("%s:fname=%s\n", __FUNCTION__, fname);
#endif

  io = g_io_channel_new_file (fname, "r", NULL);

  if (io == NULL)
  {
    g_print ("%s:file not exits\n", __FUNCTION__);
    return FALSE;
  }

  do
  {
    gchar *buffer = NULL;
    gchar **strs = NULL;
    gint i;

    status = g_io_channel_read_line (io, &buffer, NULL, NULL, NULL);
    if (!buffer)
      continue;
    strs = g_strsplit (buffer, ",", 2);

    for (i = 0; strs[i] != NULL; i++)
    {
      g_strstrip (strs[i]);
    }
    if (i >= 2)
    {
      gint id;
      gint value;


      id = atoi (strs[0]);
      value = atoi (strs[1]);

      switch (id)
      {
      case WEEK_STARTS_MONDAY:
        {
          if (value == 0)
            setting->week_starts_monday = FALSE;
          else if (value == 1)
            setting->week_starts_monday = TRUE;
          setting->week_offset = 1 - value;
        }
        break;
      case DEFAULT_VIEW:
        {
          setting->default_view = value;
        }
        break;
      case FORCE_TODAY:
        {
          if (value == 0)
            setting->force_today = FALSE;
          else if (value == 1)
            setting->force_today = TRUE;
        }
        break;
      }

    }

    g_strfreev (strs);
    g_free (buffer);

  } while (status == G_IO_STATUS_NORMAL);

  g_io_channel_shutdown (io, TRUE, NULL);
  g_io_channel_unref (io);

  return TRUE;
}

gboolean
cal_setting_write_file (CalSetting *setting, const gchar *fname)
{
  GIOChannel *io = NULL;
  GIOStatus status = G_IO_STATUS_NORMAL;


  g_assert (setting);
  g_assert (fname);


  io = g_io_channel_new_file (fname, "w", NULL);

  if (io == NULL)
  {
    g_print ("%s:file not exits\n", __FUNCTION__);
    return FALSE;
  }

  g_io_channel_seek_position (io, 0, G_SEEK_SET, NULL);

  gchar *buffer;

  buffer = g_malloc (20);
  if (setting->week_starts_monday)
    g_sprintf (buffer, "%d,%d\n", WEEK_STARTS_MONDAY, 1);
  else
    g_sprintf (buffer, "%d,%d\n", WEEK_STARTS_MONDAY, 0);

  status = g_io_channel_write_chars (io, buffer, -1, NULL, NULL);
  g_sprintf (buffer, "%d,%d\n", DEFAULT_VIEW, setting->default_view);
  status = g_io_channel_write_chars (io, buffer, -1, NULL, NULL);
  if (setting->force_today)
    g_sprintf (buffer, "%d,%d\n", FORCE_TODAY, 1);
  else
    g_sprintf (buffer, "%d,%d\n", FORCE_TODAY, 0);
  status = g_io_channel_write_chars (io, buffer, -1, NULL, NULL);
  g_free (buffer);

  g_io_channel_shutdown (io, TRUE, NULL);
  g_io_channel_unref (io);

  return TRUE;
}



static GtkWidget *
create_combo_dview (void)
{
  GtkWidget *combo;

  combo = gtk_combo_box_new_text ();
  gtk_combo_box_prepend_text (GTK_COMBO_BOX (combo), "Week view");
  gtk_combo_box_prepend_text (GTK_COMBO_BOX (combo), "Day view");
  gtk_combo_box_prepend_text (GTK_COMBO_BOX (combo), "Month view");

  gtk_combo_box_set_active (GTK_COMBO_BOX (combo), self.setting->default_view);
  return combo;
}

static GtkWidget *
create_combo_weekstart (void)
{
  GtkWidget *combo;

  combo = gtk_combo_box_new_text ();
  gtk_combo_box_prepend_text (GTK_COMBO_BOX (combo), "Sunday");
  gtk_combo_box_prepend_text (GTK_COMBO_BOX (combo), "Monday");

  gtk_combo_box_set_active (GTK_COMBO_BOX (combo), self.setting->week_offset);

  return combo;
}


gboolean
cal_save_setting (CalSettingPage *view)
{
  gchar *res_path = NULL;
  gchar *settingfile = NULL;
  guint dview, weekstart;

  res_path = getenv ("CAL_RES_PATH");
  settingfile = g_build_filename (res_path, "setting.cal", NULL);

  dview = gtk_combo_box_get_active (GTK_COMBO_BOX (view->combo_dview));
  self.setting->default_view = dview;
  weekstart = gtk_combo_box_get_active (GTK_COMBO_BOX (view->combo_weekstart));
  if (weekstart == 0)
    self.setting->week_starts_monday = TRUE;
  else if (weekstart == 1)
    self.setting->week_starts_monday = FALSE;
  self.setting->week_offset = weekstart;
  self.setting->force_today = FALSE;

  cal_setting_write_file (self.setting, settingfile);
  g_free (settingfile);

  return TRUE;
}

static void
cal_setting_page_init (CalSettingPage *view)
{


  view->vbox = gtk_vbox_new (FALSE, 0);
  view->label_dview = gtk_label_new ("Default view:");
  gtk_misc_set_alignment (GTK_MISC (view->label_dview), 0, 0);
  view->combo_dview = create_combo_dview ();

  gtk_box_pack_start (GTK_BOX (view->vbox), view->label_dview, 0, TRUE, 0);
  gtk_box_pack_start (GTK_BOX (view->vbox), view->combo_dview, 0, TRUE, 0);


  view->label_weekstart = gtk_label_new ("Weeks starts on:");
  gtk_misc_set_alignment (GTK_MISC (view->label_weekstart), 0, 0);
  view->combo_weekstart = create_combo_weekstart ();

  gtk_box_pack_start (GTK_BOX (view->vbox), view->label_weekstart, 0, TRUE, 0);
  gtk_box_pack_start (GTK_BOX (view->vbox), view->combo_weekstart, 0, TRUE, 0);


  gtk_auto_scrolled_window_add_with_viewpoert (GTK_AUTO_SCROLLED_WINDOW (view),
                                               view->vbox);

  return;
}

GtkWidget *
cal_setting_page_new (void)
{
  return gtk_widget_new (CAL_TYPE_SETTING_PAGE, NULL);
}

void
cal_setting_page_update (CalSettingPage *view)
{
  gtk_combo_box_set_active (GTK_COMBO_BOX (view->combo_dview),
                            self.setting->default_view);
  gtk_combo_box_set_active (GTK_COMBO_BOX (view->combo_weekstart),
                            self.setting->week_offset);
}

/*
vi:ts=2:nowrap:ai:expandtab
*/

