/*	calendar - calendaring and scheduling application reference implementation
 *
 *  Authors: YANG Lin <lin.yang@orange-ftgroup.com> 
 *  
 *  This software and associated documentation files (the "Software") 
 *  are copyright (C) 2005 LiPS Linux Phone Standards Forum [FranceTelecom] 
 *  All Rights Reserved. 
 *
 *  A copyright license is hereby granted for redistribution and use of 
 *  the Software in source and binary forms, with or without modification, 
 *  provided that the following conditions are met: 
 *  - Redistributions of source code must retain the above copyright notice, 
 *  this copyright license and the following disclaimer. 
 *  - Redistributions in binary form must reproduce the above copyright 
 *   notice, this copyright license and the following disclaimer in the 
 *  documentation and/or other materials provided with the distribution. 
 *  - Neither the name of LiPS  nor the names of its Members may be used 
 *  to endorse or promote products derived from the Software without 
 *  specific prior written permission. 
 *
 *  A patent license for any Necessary Claims owned by Members of LiPS Forum 
 *  to make, have made, use, import, offer to sell, lease and sell or otherwise 
 *  distribute any implementation compliant with the any specification adopted 
 *  by the LiPS Forumcan be obtained from the respective Members on reasonable 
 *  and non-discriminatory terms and conditions and under reciprocity, as 
 *  regulated in more detail in the Internal Policy of the LiPS Forum. 
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDER, ITS MEMBERS AND CONTRIBUTORS 
 *  "AS IS", AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
 *  THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE 
 *  AND NON-INFRINGEMENT ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER, 
 *  ITS MEMBERS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
 *  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; 
 *  OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 *  WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 *  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
 *  POSSIBILITY OF SUCH DAMAGE. 
 */


#include <stdio.h>
#include <string.h>

#include <gtk/gtk.h>
#include <glib.h>
#include <langinfo.h>
#include <gdk/gdkkeysyms.h>

#include "weekview.h"
#include "setting.h"
#include "timemanage.h"
#include "calenabler.h"
#include "basicframe.h"


static void cal_week_view_class_init (CalWeekViewClass *klass);
static void cal_week_view_init (CalWeekView *view);

/*extern nl_item days1[7];
extern nl_item days2[7];*/

extern gchar *days1[7];
extern gchar *days2[7];


static guint xp = 6, xs = 24, ys = 22;
static guint drawsize = 560;

GType
cal_week_view_get_type (void)
{
  static GType weekview_type = 0;

  if (!weekview_type)
  {
    static const GTypeInfo weekview_info = {
      sizeof (CalWeekViewClass),
      NULL,                     /* base_init */
      NULL,                     /* base_finalize */
      (GClassInitFunc) cal_week_view_class_init,
      NULL,                     /* class_finalize */
      NULL,                     /* class_data */
      sizeof (CalWeekView),
      0,                        /* n_preallocs */
      (GInstanceInitFunc) cal_week_view_init,
    };

    weekview_type =
      g_type_register_static (GTK_TYPE_FRAME, "CalWeekView", &weekview_info, 0);
  }

  return weekview_type;
}

static void
cal_week_view_class_init (CalWeekViewClass *klass)
{
  return;
}

gboolean
week_view_day_indicate_update (CalWeekView *view)
{
  gchar buffer[255];
  struct tm tm;
  time_t viewtime = cal_get_viewtime ();

  localtime_r (&viewtime, &tm);
  strftime (buffer, sizeof (buffer), WEEK_TIME_FMT, &tm);
  gtk_label_set_text (GTK_LABEL (view->date_indicate), buffer);

  return TRUE;
}

static gint
week_view_draw_expose_cb (GtkWidget *widget,
                          GdkEventExpose *event, gpointer user_data)
{
  CalWeekView *view = (CalWeekView *) user_data;
  GtkDrawingArea *darea;
  GdkDrawable *drawable;
  guint width, height;

  GdkGC *black_gc;
  GdkGC *gray_gc;
  GdkGC *white_gc;
  GdkGC *cream_gc;
  GdkGC *light_gray_gc;
  GdkGC *yellow_gc;
  GdkGC *salmon_gc;
  GdkGC *blue1_gc;              /*light blue, for the normal day */
  GdkGC *blue2_gc;              /*dark blue, for today */
  GdkGC *blue_gc;

  GdkColor cream;
  GdkColor light_gray;
  GdkColor yellow;
  GdkColor salmon;
  GdkColor blue;
  GdkColor blue1;
  GdkColor blue2;

  GdkColormap *colormap;
  guint i, j;
  PangoLayout *pl;
  PangoLayout *pl_evt;
  PangoRectangle pr;
  /*nl_item *days;*/
  gchar **days;
  g_return_val_if_fail (widget != NULL, TRUE);
  g_return_val_if_fail (GTK_IS_DRAWING_AREA (widget), TRUE);
#ifdef DEBUG
  printf ("entering %s\n", __FUNCTION__);
#endif
  days = (self.setting->week_starts_monday) ? days1 : days2;

  colormap = gdk_window_get_colormap (widget->window);

  cream_gc = gdk_gc_new (widget->window);
  gdk_gc_copy (cream_gc, widget->style->black_gc);
  cream.red = 65535;
  cream.green = 60138;
  cream.blue = 60138;
  gdk_colormap_alloc_color (colormap, &cream, FALSE, TRUE);
  gdk_gc_set_foreground (cream_gc, &cream);

  light_gray_gc = gdk_gc_new (widget->window);
  gdk_gc_copy (light_gray_gc, widget->style->black_gc);
  light_gray.red = 53040;
  light_gray.green = 53040;
  light_gray.blue = 53040;
  gdk_colormap_alloc_color (colormap, &light_gray, FALSE, TRUE);
  gdk_gc_set_foreground (light_gray_gc, &light_gray);

  blue_gc = gdk_gc_new (widget->window);
  gdk_gc_copy (blue_gc, widget->style->black_gc);
  blue.red = 14906;
  blue.green = 35980;
  blue.blue = 55769;
  gdk_colormap_alloc_color (colormap, &blue, FALSE, TRUE);
  gdk_gc_set_foreground (blue_gc, &blue);

  blue1_gc = gdk_gc_new (widget->window);
  gdk_gc_copy (blue1_gc, widget->style->black_gc);
  blue1.red = 53199;
  blue1.green = 59110;
  blue1.blue = 65535;
  gdk_colormap_alloc_color (colormap, &blue1, FALSE, TRUE);
  gdk_gc_set_foreground (blue1_gc, &blue1);

  blue2_gc = gdk_gc_new (widget->window);
  gdk_gc_copy (blue2_gc, widget->style->black_gc);
  blue2.red = 36237;
  blue2.green = 47802;
  blue2.blue = 58596;
  gdk_colormap_alloc_color (colormap, &blue2, FALSE, TRUE);
  gdk_gc_set_foreground (blue2_gc, &blue2);

  yellow_gc = gdk_gc_new (widget->window);
  gdk_gc_copy (yellow_gc, widget->style->black_gc);
  gdk_color_parse ("palegoldenrod", &yellow);
  gdk_colormap_alloc_color (colormap, &yellow, FALSE, TRUE);
  gdk_gc_set_foreground (yellow_gc, &yellow);

  salmon_gc = gdk_gc_new (widget->window);
  gdk_gc_copy (salmon_gc, widget->style->black_gc);
  gdk_color_parse ("light salmon", &salmon);
  gdk_colormap_alloc_color (colormap, &salmon, FALSE, TRUE);
  gdk_gc_set_foreground (salmon_gc, &salmon);


  white_gc = widget->style->white_gc;
  gray_gc = widget->style->bg_gc[GTK_STATE_NORMAL];
  black_gc = widget->style->black_gc;

  gdk_gc_set_clip_rectangle (black_gc, &event->area);
  gdk_gc_set_clip_rectangle (gray_gc, &event->area);
  gdk_gc_set_clip_rectangle (white_gc, &event->area);
  gdk_gc_set_clip_rectangle (cream_gc, &event->area);
  gdk_gc_set_clip_rectangle (light_gray_gc, &event->area);
  gdk_gc_set_clip_rectangle (yellow_gc, &event->area);
  gdk_gc_set_clip_rectangle (salmon_gc, &event->area);
  gdk_gc_set_clip_rectangle (blue_gc, &event->area);
  gdk_gc_set_clip_rectangle (blue1_gc, &event->area);
  gdk_gc_set_clip_rectangle (blue2_gc, &event->area);

  pl = gtk_widget_create_pango_layout (GTK_WIDGET (widget), NULL);
  pl_evt = gtk_widget_create_pango_layout (GTK_WIDGET (widget), NULL);

  darea = GTK_DRAWING_AREA (widget);
  drawable = widget->window;

  width = widget->allocation.width;
  height = widget->allocation.height;

  gdk_window_clear_area (drawable,
                         event->area.x, event->area.y,
                         event->area.width, event->area.height);


  for (i = 0; i < 8; i++)
  {
    guint x = xp + (i * xs);
    guint w;
    gchar *s;

    /*Write the week day in title part */
    if (i > 0)
    {
      /*s  = g_locale_to_utf8 (nl_langinfo (days[i-1]), -1,
         NULL, NULL, NULL); */
      s = g_strdup (days[i - 1]);

      pango_layout_set_text (pl, s, strlen (s));
      pango_layout_get_pixel_extents (pl, &pr, NULL);
      w = pr.width;


      if ((i > 6) || (i < 2))
        gdk_draw_rectangle (drawable, cream_gc, TRUE, x, 0, xs, ys);
      else
        gdk_draw_rectangle (drawable, blue1_gc, TRUE, x, 0, xs, ys);


      gtk_paint_layout (widget->style,
                        widget->window,
                        GTK_WIDGET_STATE (widget),
                        FALSE,
                        &event->area, widget, "label", x + (xs - w) / 2, 2, pl);
    }

    /*Draw the hour indicator */
    else                        
    {
      for (j = 1; j < 25; j++)
      {
        gchar *buf = NULL;

        buf = g_strdup_printf ("%2d", j - 1);
        pango_layout_set_text (pl, buf, strlen (buf));
        pango_layout_get_pixel_extents (pl, &pr, NULL);


        gtk_paint_layout (widget->style,
                          widget->window,
                          GTK_WIDGET_STATE (widget),
                          FALSE,
                          &event->area, widget, "label", 0, j * ys + 2, pl);
        /*g_free (buf);*/
      }

    }

  }

  for (i = 0; i < 7; i++)
    for (j = 0; j < 24; j++)
    {
      guint x = xp + ((i + 1) * xs);
      guint y = (j + 1) * ys;
      guint d = i * 24 + j;
      struct day_render_ctl *c = &(view->rc[d]);


      if (c->events)
      {
        gdk_draw_rectangle (drawable, blue_gc, TRUE, x, y, xs, ys);
#ifdef DEBUG
        printf ("%s: c[%d] should has events\n", __FUNCTION__, d);
#endif
      }

    }


  /*Draw the grid */
  for (i = 0; i < 8; i++)
  {
    guint x;

    if (i == 0)
      x = 0;
    else
      x = xp + (i * xs);

    for (j = 0; j < 25; j++)
    {
      guint y = j * ys;
      if (i != 0)
        gdk_draw_rectangle (drawable, light_gray_gc, FALSE, x, y, xs, ys);
      else
        gdk_draw_rectangle (drawable, light_gray_gc, FALSE, x, y, xs + xp, ys);
    }
  }
  /*    
     for (i = 0; i < 7; i++)
     {
     guint x = xp + (i * xs);

     for (j = 0; j < (TOTAL_MONTH_DAYS / 7); j++)
     {
     guint d = i + (7 * j) ;
     struct render_ctl *c = &(view->rc[d]);
     guint y = (j+1) * ys;

     if (!(cal_get_force_today()) && c->active) 
     {
     gdk_draw_rectangle (drawable, blue_gc, FALSE,
     x, y, xs, ys);
     gdk_draw_rectangle (drawable, blue_gc, FALSE,
     x + 1, y + 1, xs - 2, ys - 2);


     }

     if (!(cal_get_force_today()) && c->active) 
     {

     if (c->events)
     {
     cal_event_t *ev;
     gchar *string;
     gchar *description;
     time_t start;
     GSList *iter;                          
     char starttime[20];                  


     for (iter = c->events; iter; iter = iter->next)
     {
     struct tm tm;

     ev = iter->data;
     description = event_get_description (ev);
     start = event_get_start (ev);
     localtime_r (&start, &tm);

     strftime (starttime, sizeof (starttime), "%F %R", &tm);

     string = g_strconcat (starttime, description, NULL);

     gtk_label_set_text (GTK_LABEL(view->labeltime1), string);        

     }                      
     g_free (description);
     g_free (string);
     }
     }
     }
     }

   */
  gdk_gc_set_clip_rectangle (black_gc, NULL);
  gdk_gc_set_clip_rectangle (gray_gc, NULL);
  gdk_gc_set_clip_rectangle (white_gc, NULL);

  gdk_gc_unref (blue_gc);
  gdk_gc_unref (blue1_gc);
  gdk_gc_unref (blue2_gc);
  gdk_gc_unref (cream_gc);
  gdk_gc_unref (light_gray_gc);
  gdk_gc_unref (yellow_gc);
  gdk_gc_unref (salmon_gc);


  g_object_unref (pl);

  return TRUE;
}

void
cal_week_view_update (CalWeekView *view)
{
  time_t day = cal_get_viewtime ();
  struct tm tm;
  guint i, j;

  localtime_r (&day, &tm);

  if (self.setting->week_starts_monday)
    tm.tm_wday = (tm.tm_wday + 6) % 7;

  /*First day */
  day -= SECONDS_IN_DAY * tm.tm_wday;
  day -= tm.tm_sec + (60 * tm.tm_min) + (60 * 60 * tm.tm_hour);



  for (i = 0; i < 7; i++)
  {
    if (view->week_events[i])
      cal_event_list_destroy (view->week_events[i]);
    view->week_events[i] = cal_event_list_init (day, day + SECONDS_IN_DAY - 1);



    for (j = 0; j < 24; j++)
    {
      struct day_render_ctl *c = &(view->rc[i * 24 + j]);


      localtime_r (&day, &tm);

      c->day = tm.tm_mday;
      c->month = tm.tm_mon;
      c->year = tm.tm_year;
      c->hour = tm.tm_hour;

      if (c->events)
        c->events = NULL;

      day += SECONDS_IN_HOUR;

    }

    if (view->week_events[i])
    {
      GSList *iter;
      cal_event_t *ev;
      time_t start, end;

      for (iter = view->week_events[i]; iter; iter = iter->next)
      {
        struct tm tm_start, tm_end;

        ev = iter->data;
        start = event_get_start (ev);
        localtime_r (&start, &tm_start);
        end = event_get_end (ev);
        localtime_r (&end, &tm_end);
#ifdef DEBUG
        /*printf ("start = %d, end = %d\n", start, end);*/
#endif
        for (j = 0; j < 24; j++)
        {
          struct day_render_ctl *c = &(view->rc[i * 24 + j]);

          if ((c->hour <= tm_end.tm_hour) && (c->hour >= tm_start.tm_hour))
          {
            c->events = g_slist_append (c->events, ev);
#ifdef DEBUG
            /*printf ("%s:c[%d] has events\n", __FUNCTION__ ,i*24+j);*/
#endif
          }
        }
      }
    }
  }
  week_view_day_indicate_update (view);
}


static gboolean
week_view_key_press_event_cb (GtkWidget *widget, GdkEventKey *k,
                              GtkWidget *user_data)
{
  CalWeekView *view = (CalWeekView *) user_data;
  GtkAdjustment *vadj =
    gtk_scrolled_window_get_vadjustment (GTK_SCROLLED_WINDOW (view->scroller));

  gdouble value = gtk_adjustment_get_value (vadj);

  switch (k->keyval)
  {
  case GDK_Return:
    break;
  case GDK_Down:
    {
      if (value < 320)
        gtk_adjustment_set_value (vadj, value + 20);
    }
    break;
  case GDK_Up:
    {
      if (value >= 20)
        gtk_adjustment_set_value (vadj, value - 20);
    }
    break;

  }
  return TRUE;
}


static void
cal_week_view_init (CalWeekView *view)
{
  guint i;

/*Set up the widget*/
  view->vbox = gtk_vbox_new (FALSE, 0);

  view->date_indicate = gtk_label_new ("day indicate");
  gtk_box_pack_start (GTK_BOX (view->vbox), view->date_indicate, FALSE, FALSE,
                      0);


  view->scroller = gtk_scrolled_window_new (NULL, NULL);


  view->draw = gtk_drawing_area_new ();

  gtk_widget_set_size_request (view->draw, 300, drawsize);
  g_signal_connect (G_OBJECT (view->draw), "expose_event",
                    G_CALLBACK (week_view_draw_expose_cb), view);

  gtk_scrolled_window_add_with_viewport (GTK_SCROLLED_WINDOW (view->scroller),
                                         view->draw);
  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (view->scroller),
                                  GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);
  gtk_box_pack_start (GTK_BOX (view->vbox), view->scroller, TRUE, TRUE, 0);


  g_signal_connect (G_OBJECT (view->draw), "key_press_event",
                    G_CALLBACK (week_view_key_press_event_cb), view);

  GTK_WIDGET_SET_FLAGS (view->draw, GTK_CAN_FOCUS);
  /* GTK_WIDGET_SET_FLAGS(view->scroller, GTK_CAN_FOCUS);*/
  gtk_widget_add_events (GTK_WIDGET (view->draw),
                         GDK_KEY_PRESS_MASK | GDK_KEY_RELEASE_MASK);
/*  gtk_widget_add_events (GTK_WIDGET (view->scroller),
            GDK_KEY_PRESS_MASK | GDK_KEY_RELEASE_MASK); */



  for (i = 0; i < 7 * 24; i++)
  {

    view->rc[i].active = FALSE;
    view->rc[i].events = NULL;
  }



/*Day events*/
  for (i = 0; i < 7; i++)
    view->week_events[i] = NULL;

  cal_week_view_update (view);

  gtk_container_add (GTK_CONTAINER (view), GTK_WIDGET (view->vbox));

  return;
}

GtkWidget *
cal_week_view_new (void)
{
  return gtk_widget_new (CAL_TYPE_WEEK_VIEW, NULL);
}

void
cal_week_view_destroy (CalWeekView * view)
{
  gint day;

  for (day = 0; day < 7; day++)
    if (view->week_events[day])
    {
      cal_event_list_destroy (view->week_events[day]);
      view->week_events[day] = NULL;
    }
}

/*
vi:ts=2:nowrap:ai:expandtab
*/

